/*
 * Created on 2005-03-05
 *
 */
package vr.data;

import java.util.Map;

import rseslib.structure.attribute.Header;
import rseslib.structure.data.DoubleDataTable;
import vr.visual.dataIO.ClassificationResultsOutput;

/**
 * Klasa przechowujaca wyniki ostatniej klasyfikacji oraz dane, na ktorych byla ona przeprowadzona.
 * 
 * @author Michal Manski, Marcin Staszczyk
 * 
 */
public class LastClassificationInfo {
	/** wyniki ostatniej klasyfikacji */
	static private Map results = null;
	/** naglowek danych dla ostatniej klasyfikacji */
	static private Header header = null;
	/** dane testowe uzyte w ostatniej klasyfikacji */
	static private DoubleDataTable testTable = null;
	/** dane treningowe uzyte w ostatniej klasyfikacji */
	static private DoubleDataTable trainTable = null;

	
	/**
     * Przekazuje naglowek oraz dane testowe i treningowe
     *
     * @param newHeader     naglowek danych.
     * @param newTestTable  dane testowe
     * @param newTrainTable dane treningowe
     */
	static public void addData(Header newHeader, DoubleDataTable newTestTable, 
			DoubleDataTable newTrainTable) {
		header = newHeader;
		testTable = newTestTable;
		trainTable = newTrainTable;
		
	}
	
	/**
     * Dodaje wyniki klasyfikacji multiklasyfikatora.
     *
     * @param newResults  wyniki klasyfikacji.
     */
	static public void addResults(Map newResults) {
		results = newResults;
	}
	
	/**
     * Zapisuje szczegoly ostatniej klasyfikacji do pliku.
     */
	static public void writeResults(){
		new ClassificationResultsOutput(ClassificationResultsOutput.FILE,
					header, trainTable, testTable, results);
	}
	
	/**
	 * Zwraca wyniki ostatniej klasyfikacji
	 *  
	 * @return wyniki klasyfikacji jako obiekt typu Map
	 */
	static public Map getResults() {
		return results;
	}
	
	/**
	 * Zwraca naglowek uzyty w ostatniej klasyfikacji
	 *  
	 * @return naglowek danych
	 */
	static public Header getHeader() {
		return header;
	}
}
