/*
 * Created on 2005-03-10
 *
 */
package vr.processing.classification;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import rseslib.processing.classification.Classifier;
import rseslib.processing.classification.MultiClassifier;
import rseslib.processing.classification.SingleClassifierTest;
import rseslib.processing.classification.TestResult;
import rseslib.structure.data.DoubleDataTable;
import rseslib.system.progress.Progress;
import rseslib.system.progress.StdOutProgress;
import vr.system.classifiersManagement.ClassifierRegistrator;
import vr.system.options.StandaloneSavableConfiguration;
import vr.system.progress.VisualStdOutMultiProgress;

/**
 * Klasa odpowiedzialna za zarzadzanie dowolna iloscia klasyfikatorow
 * tzn.:
 * 1. tworzenie instancji klas na podstawie obiektu Class
 * 2. uczy klasyfikatory
 * 3. testuje klasyfikatory i zwraca mape wynikow
 * Zaklada, ze podawane nazwy nie powtarzaja sie
 *
 *
 * @author Marcin Staszczyk
 *
 */
public class GeneralMultiClassifier {

	/** Mapa pomiedzy nazwami klasyfikatorow a ich obiektami typu Class. */
	private Map c_classifiers = new HashMap();
	/** Mapa pomiedzy nazwami multiklasyfikatorow a ich obiektami typu Class. */
	private Map c_multiClassifiers = new HashMap();

	/** Mapa pomiedzy nazwami klasyfikatorow a ich opcjami. */
	private Map p_classifiers = new HashMap();
	/** Mapa pomiedzy nazwami multiklasyfikatorow a ich opcjami. */
	private Map p_multiClassifiers = new HashMap();

	/** Mapa pomiedzy nazwami klasyfikatorow a ich instancjami. */
    private Map o_Classifiers = new HashMap();
	/** Mapa pomiedzy nazwami multiklasyfikatorow a ich instancjami. */
    private Map o_MultiClassifiers = new HashMap();

    /** Progres w uczeniu i testowaniu klasyfikatorow */
    private VisualStdOutMultiProgress prog;

    /** Informauje, czy podczas klasyfikacji wystapily jakies blady */
    public boolean errors_appeared = false;

    /** Klasyfikator pojedynczej tabeli */
    public SingleClassifierTest tester = new SingleClassifierTest();

    /**
	 * Zwykly konstruktor klasy.
	 * Narazie nie robi nic oprocz wczytania domyslnych opcji
	 * Klasa nie uzywa puki co opcji, ale w przypadku rozwoju sa calkiem prawdopodobne
	 */
	public GeneralMultiClassifier() {
	}

	/**
	 * Zwykly konstruktor klasy.
	 * Narazie nie robi nic oprocz wczytania opcji
	 * Klasa nie uzywa puki co opcji
	 * DO PROJEKTU N I E  U Z Y W A C
	 *
	 * @param prop
	 */
	public GeneralMultiClassifier(Properties prop) {
	}

	/**
	 * Funkcja dodajaca klasyfikator do ogolnego multiklasyfikatora.
	 * Zaklada ze podawane dane sa poprawne, tzn.:
	 * 1. flagi odpowiadaja klasyfikatorowi
	 * 2. podawane klasyfikatory potrafia sobie poradzic z danymi na ktorych zostanie wywolany multiklasyfikator
	 * 3. nazwy sie nie powtarzaja
	 *
	 * @param classifier Klasyfikator ktory chcemy dodac
	 * @param flags Flagi dodawanego klasyfikatora
	 */
	public void addNewClassifier(Class classifier, int flags, String name, StandaloneSavableConfiguration conf)
	{
		if ((flags & ClassifierRegistrator.FLAG_MULTICLASSIFIER) == 0)
		{
			c_classifiers.put(name, classifier);
			p_classifiers.put(name, (conf == null ? null : conf.getProperties()));
		}
		else
		{
			c_multiClassifiers.put(name, classifier);
			p_multiClassifiers.put(name, (conf == null ? null : conf.getProperties()));
		}
	}

	/**
	 * Funkcja dodajaca klasyfikator do ogolnego multiklasyfikatora.
	 * Zaklada ze podawane dane sa poprawne, tzn.:
	 * 1. flagi odpowiadaja klasyfikatorowi
	 * 2. podawane klasyfikatory potrafia sobie poradzic z danymi na ktorych zostanie wywolany multiklasyfikator
	 *
	 * @param classifier 	Klasyfikator ktory chcemy dodac
	 * @param flags 		Flagi dodawanego klasyfikatora
	 */
	public void addNewClassifier(Class classifier, int flags)
	{
		if ((flags & ClassifierRegistrator.FLAG_MULTICLASSIFIER) == 0)
		{
			c_classifiers.put(classifier.getName(), classifier);
			p_classifiers.put(classifier.getName(), null);
		}
		else
		{
			c_multiClassifiers.put(classifier.getName(), classifier);
			p_multiClassifiers.put(classifier.getName(), null);
		}
	}

	/**
	 * Uczy podane wczesniej klasyfikatory danymi z tabeli
	 *
	 * @param trainTable				Dane treningowe
	 * @throws InterruptedException		W przypadku przerwania przez uzytkownika klasyfikacji zwracamy blad
	 */
	public void train(DoubleDataTable trainTable) throws InterruptedException
	{
		Progress trainingProgress;

		Class[] args_t = new Class[3];
		args_t[0] = Properties.class;
		args_t[1] = DoubleDataTable.class;
		args_t[2] = Progress.class;

		prog = new VisualStdOutMultiProgress();
        prog.mainSet("Klasyfikacja", 2*(c_classifiers.size() + c_multiClassifiers.size()));

		Iterator itr = c_classifiers.keySet().iterator();
		while (itr.hasNext())
		{
			String name = (String)itr.next();
			Class classifier = (Class)c_classifiers.get(name);
			Properties prop = (Properties)p_classifiers.get(name);
			Object[] args = {prop, trainTable, prog};
			try
			{
				o_Classifiers.put(name, (Classifier)(classifier.getConstructor(args_t)).newInstance(args));
			}
			catch (Exception e)
			{
				prog.mainStep();/* bo w fazie testowania nie zostanie on policzony */
				try
				{
					errors_appeared = true;
					throw new RuntimeException("Blad podczas tworzenia lub uczenia klasyfikatora " + classifier.getName() + '\n' + e);
				}
				catch(RuntimeException re) {}
			}
			prog.mainStep();
		}

		args_t = new Class[1];
		args_t[0] = Properties.class;
		itr = c_multiClassifiers.keySet().iterator();
		while (itr.hasNext())
		{
			String name = (String)itr.next();
			Class classifier = (Class)c_multiClassifiers.get(name);
			Properties prop = (Properties)p_multiClassifiers.get(name);
			Object[] args = {prop};
			try
			{
				MultiClassifier abMulCl = (MultiClassifier)(classifier.getConstructor(args_t)).newInstance(args);
				abMulCl.classify(trainTable, new StdOutProgress());
				o_MultiClassifiers.put(name, abMulCl);
			}
			catch (Exception e)
			{
				prog.mainStep();/* bo w fazie testowania nie zostanie on policzony */
				try
				{
					errors_appeared = true;
					throw new RuntimeException("Blad podczas tworzenia lub uczenia klasyfikatora " + classifier.getName() + '\n' + e);
				}
				catch(RuntimeException re) {}
			}
			prog.mainStep();
		}
	}

	/**
	 * Testuje nauczone wczesniej klasyfikatory na podstawie podanych danych testowych
	 *
	 * @param tstTable 				dane testowe
	 * @return						Mapa (nazwy nadane klasyfikatorom -> TestResult)
	 * @throws InterruptedException
	 */
	public Map classify(DoubleDataTable tstTable) throws InterruptedException
    {
        Map resultsMap = new HashMap();
        Set classifierNames = o_Classifiers.keySet();
        Iterator dataIter = tstTable.iterator();
        int tstDataNumber = 0;

        Iterator classifierIter = classifierNames.iterator();
    	while (classifierIter.hasNext())
        {
    		String classifierName = (String)classifierIter.next();
    		try
			{
    			TestResult result = tester.classify((Classifier)o_Classifiers.get(classifierName), tstTable, prog);
    			resultsMap.put(classifierName, result);
			}
    		catch(Exception e)
			{
    			try
				{
					errors_appeared = true;
					throw new RuntimeException("Blad podczas tworzenia lub uczenia klasyfikatora " + classifierName + '\n' + e);
				}
				catch(RuntimeException re) {}
			}
            prog.mainStep();
        }

        classifierNames = o_MultiClassifiers.keySet();
        classifierIter = classifierNames.iterator();
    	while (classifierIter.hasNext())
        {
            String classifierName = (String)classifierIter.next();
            try
			{
            	Map dec = ((MultiClassifier)o_MultiClassifiers.get(classifierName)).classify(tstTable,prog);
            	resultsMap.put(classifierName, dec);
			}
    		catch(Exception e)
			{
    			try
				{
					errors_appeared = true;
					throw new RuntimeException("Blad podczas tworzenia lub uczenia klasyfikatora " + classifierName + '\n' + e);
				}
				catch(RuntimeException re) {}
			}
            prog.mainStep();
        }

        return resultsMap;
    }
}
