/*
 * Created on 2005-03-22
 *
 */
package vr.system.options;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Enumeration;
import java.util.Properties;


/**
 * Klasa odpowiedzialna za zarzadzanie plikami z domyslnymi opcjami class
 * bez potrzeby istnienia ich instancji.
 * Umozliwia:
 * 1. Wczytanie opcji z pliku konfiguracyjnego
 * 2. Zapisanie opcji do pliku konfiguracyjnego
 *
 * @author Marcin Staszczyk
 */
public class StandaloneSavableConfiguration extends SavableConfiguration {

	private String propFile;

	/**
	 * Konstruktor przygotowyujacy wszyskie potrzebne dane.
	 * Wczytujacy domyslne ustawienia dla podanej klasy.
	 * Znajduje sciezke do pliku z ustawieniami domslnymi
	 *
	 * @param configurableClass 	Klasa dla ktorej ustawiamy opcje
	 */
	public StandaloneSavableConfiguration(Class configurableClass) {
		super(loadDefaultProperties(configurableClass));
		propFile = getDefaultProperitesFile(configurableClass);
	}


	/**
     * Wczytuje domyslne opcje dla podanej klasy.
     *
     * @param configurableClass       Klasa.
     * @return Properties             Domyslne ustawienia dla podanej klasy.
     */
    public static Properties loadDefaultProperties(Class configurableClass)
    {
        Properties prop = new Properties();
        String[] pathes = getPropertiesFilePathes(configurableClass.getName());
        for (int path = 0; path < pathes.length; path++)
        {
            File propertyFile = new File(pathes[path]);
            try
            {
                if (propertyFile.exists()) prop.load(new BufferedInputStream(new FileInputStream(propertyFile)));
            }
            catch (IOException e)
            {
                throw new RuntimeException("Error while reading properties file: "+e.getMessage());
            }
        }
        return prop;
    }

    /**
     * Zwraca domyslna sciezke do pliku z konfiguracja podanej klasy
     *
     * @param configurableClass		Klasa.
     * @return String				Domyslna sciezka do pliku z konfiguracja podanej klasy
     */
    protected static String getDefaultProperitesFile(Class configurableClass)
    {
    	String[] prop = getPropertiesFilePathes(configurableClass.getName());

    	return prop[0];
    }

    /**
     * Zwraca jakie nazwy sa zdefiniowane
     *
     * @return Iterator po nazwach (String)
     */
    public Enumeration getPropertiesNames()
    {
    	return m_Properties.propertyNames();
    }

    /**
     * Zwraca obiekt przechwujacy opcje,
     * dla ktorego klasa StandaloneSavableConfiguration jest opakowujaca
     *
     * @return 	obiekt przechowujacy opcje
     */
    public Properties getProperties()
    {
    	return m_Properties;
    }

    /**
     * Zapisuje opcje do domyslnego pliku
     */
    public void writeProperties()
    {
    	try {
        	writeProperties(new PrintStream(new FileOutputStream(propFile)));
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Blad. Nie powinno byc problemow\n" + e);
		}
    }
}
