/*
 * Created on 2005-02-27
 *
 */
package vr.system.progress;

import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.Spring;
import javax.swing.SpringLayout;

import rseslib.system.Report;
import rseslib.system.progress.AbstractProgress;
/**
 * Wyswietla informacje o postpie w nowym oknie.
 * Przystosowana do jednego glownego progresu i wielu podrzednych.
 *
 * @author Micha Maski i Marcin Staszczyk
 *
 */
public class VisualStdOutMultiProgress extends AbstractProgress implements ActionListener
{
	/** Aktualny gwny krok tego "progressu". */
    int m_nCurrentPoint;
    /** Aktualny krok tego "progressu". */
    int m_nMinorCurrentPoint;
    /** Czas kiedy zaczo si liczenie postpu. */
    long m_nStartTime;
    /** Gowny pasek postpu. */
    JProgressBar progressBar;
    /** Pasek postpu. */
    JProgressBar minorProgressBar;
    /** "TextArea" do wyswietlania informacji. */
    JTextArea textArea;
    /** Do wyswietlania kolejnych nazw klasyfikatorw */
    JLabel minorLabel = new JLabel("");
    /** Do przerywania procesu klasyfikacji */
    boolean bInterrupt = false;
    /** Okno w ktorym wyswietlamy progres */
    JFrame frame;

    /**
     * Ustawia laczna liczbe glownych krokow do wykonania.
     *
     * @param name      Nazwa tego "progressu".
     * @param noOfSteps Liczba krokw.
     */
    public void mainSet(String name, int noOfSteps)
    {
        super.set(name, noOfSteps);
        m_nCurrentPoint = 0;

        frame = new JFrame("VRWindow postepu");

        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        LayoutManager m = new SpringLayout();
        frame.getContentPane().setLayout(m);

        progressBar = new JProgressBar(0, noOfSteps);
        progressBar.setValue(0);
        progressBar.setStringPainted(true);
        JLabel label = new JLabel(m_Name+":");
        textArea = new JTextArea(5,20);
        textArea.setMargin(new Insets(5,5,5,5));
        textArea.setEditable(false);
        textArea.setCursor(null);

        minorProgressBar = new JProgressBar(0, 1);
        minorProgressBar.setValue(0);
        minorProgressBar.setStringPainted(true);

        JButton stopButton = new JButton("Przerwij");
        stopButton.addActionListener(this);

        frame.getContentPane().add(label, new SpringLayout.Constraints(
        	    Spring.constant(10),Spring.constant(10),
			    Spring.constant(180),Spring.constant(25)));
        frame.getContentPane().add(progressBar, new SpringLayout.Constraints(
        	    Spring.constant(210),Spring.constant(10),
			    Spring.constant(200),Spring.constant(25)));

        frame.getContentPane().add(minorLabel, new SpringLayout.Constraints(
        	    Spring.constant(10),Spring.constant(50),
			    Spring.constant(180),Spring.constant(25)));
        frame.getContentPane().add(minorProgressBar, new SpringLayout.Constraints(
        	    Spring.constant(210),Spring.constant(50),
			    Spring.constant(200),Spring.constant(25)));

        frame.getContentPane().add(new JScrollPane(textArea), new SpringLayout.Constraints(
        	    Spring.constant(10),Spring.constant(90),
			    Spring.constant(400),Spring.constant(60)));

        frame.getContentPane().add(stopButton, new SpringLayout.Constraints(
        	    Spring.constant(150),Spring.constant(160),
			    Spring.constant(100),Spring.constant(25)));

        frame.pack();
        frame.setSize(430,230);
        frame.setVisible(true);
        m_nStartTime = System.currentTimeMillis();
    }

    /**
     * Wykonuje pojedynczy glowny krok.
     *
     * @throws InterruptedException
     */
    public void mainStep() throws InterruptedException
    {
    	if (bInterrupt)
    	{
    		frame.dispose();
    		throw new InterruptedException();
    	}

        m_nCurrentPoint++;
        progressBar.setValue(m_nCurrentPoint);
        minorProgressBar.setValue(minorProgressBar.getMaximum());

        if (m_nCurrentPoint==m_nEndPoint)
        {
            long durationTime = System.currentTimeMillis() - m_nStartTime;
            Report.displaynl("Czas klasyfikacji: " + (durationTime/1000) + "s " +  (durationTime%1000) + "ms");
            frame.setVisible(false);
            frame.dispose();
        }
    }

    /**
     * Ustawia laczna liczbe zwyklych krokow do wykonania.
     *
     * @param name      Nazwa tego "progressu".
     * @param noOfSteps Liczba krokw.
     */
    public void set(String name, int noOfSteps)
    {
    	m_nMinorCurrentPoint = 0;

    	minorProgressBar.setMaximum(noOfSteps);
    	minorProgressBar.setValue(0);
    	minorLabel.setText(name + ":");
    }

    /**
     * Wykonuje pojedynczy zwykly krok.
     *
     * @throws InterruptedException
     */
    public void step() throws InterruptedException
    {
    	if (bInterrupt)
    	{
    		frame.dispose();
    		throw new InterruptedException();
    	}
    	m_nMinorCurrentPoint++;
    	minorProgressBar.setValue(m_nMinorCurrentPoint);
    }

    /**
     * Reakcja na wcisniecie przycisku.
     */
	public void actionPerformed(ActionEvent arg0) {
		bInterrupt = true;
	}

}
