/*
 * Created on Apr 10, 2005
 *
 */
package vr.visual;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import vr.system.DataManager;
import vr.system.classifiersManagement.ClassifierInfo;
import vr.system.classifiersManagement.ClassifierInstanceInfo;
import vr.system.options.StandaloneSavableConfiguration;

/**
 * Klasa umozliwiajaca wybor klasyfikatora. Do wyboru dostepne sa tylko 
 * te klasyfikatory, ktore radza sobie z aktualnie wczytanymi danymi.
 * 
 * @author Michal Manski, Damian Manski, Tomek Romanczuk
 *
 */
public class ClassifierChooser extends JPanel implements ActionListener {
	
	/** przycisk usuniecia klasyfikatora ze zbioru wybraych */
	JButton removeButton;
	/** przycisk wybrania klasyfikatora */
	JButton addButton;
	/** przycisk opcji */
	JButton optionsButton;
	/** komponent graficzny (lista) zawierajacy dostepne klasyfikatory */
	JList clasList;
	/** komponent graficzny (lista) zawierajacy wybrane klasyfikatory */
	JList chosenList;
	/** wybrane klasyfikatory */
	ArrayList chosen = DataManager.chosenClassifiers;
	/** dostepne klasyfikatory do wyboru */
	ClassifierInfo[] info;
	/** okno */
	JFrame frame;
	
	/**
	 * Konstruktor klasy. 
	 * Tworzone jest okno pozwalajace uzytkownikowi na wybor klasyfikatorow.
	 *
	 */
	public ClassifierChooser()
	{
		super(new BorderLayout());
		
		info = DataManager.classifiersManager.getClassifiersNames();
		clasList = new JList(info);
		clasList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		chosenList = new JList(chosen.toArray());
		chosenList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JPanel buttonPanel = new JPanel();
		optionsButton = new JButton("Opcje klasyfikatora");
		optionsButton.addActionListener(this);
		buttonPanel.add(optionsButton);
		removeButton = new JButton("Usun klasyfikator");
		removeButton.addActionListener(this);
		buttonPanel.add(removeButton);
		JPanel chosenPanel = new JPanel(new BorderLayout());
		JLabel label = new JLabel("Wybrane klasyfikatory:");
		JPanel labelPanel = new JPanel();
		labelPanel.add(label);
		chosenPanel.add(labelPanel, BorderLayout.NORTH);
		chosenPanel.add(new JScrollPane(chosenList), BorderLayout.CENTER);
		chosenPanel.add(buttonPanel, BorderLayout.SOUTH);
		
		JPanel buttonPanel2 = new JPanel();
		addButton = new JButton("Dodaj klasyfikator");
		addButton.addActionListener(this);
		buttonPanel2.add(addButton);
		JPanel bottomPanel = new JPanel(new BorderLayout());
		label = new JLabel("Dostepne klasyfikatory:");
		labelPanel = new JPanel();
		labelPanel.add(label);
		bottomPanel.add(labelPanel, BorderLayout.NORTH);
		bottomPanel.add(new JScrollPane(clasList), BorderLayout.CENTER);
		bottomPanel.add(buttonPanel2, BorderLayout.SOUTH);
		bottomPanel.setBorder(BorderFactory.createEmptyBorder(45, 0, 0, 0));
		
		add(chosenPanel, BorderLayout.CENTER);
		add(bottomPanel, BorderLayout.SOUTH);
		
		setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		
		
		frame = new JFrame("Wybor klasyfikatorow");
		frame.setContentPane(this);
		frame.pack();
		frame.setVisible(true);
		frame.setLocationRelativeTo(VRWindow.mainWindow);
		
	
	}

	/**
	 * Obsluga akcji uzytkownika.
	 */
	public void actionPerformed(ActionEvent e) {

		JButton button = (JButton) e.getSource();
		if (button == addButton) {
			/* Wybieramy klasyfikator */
			int selected = clasList.getSelectedIndex();
			if (selected >= 0) {
				Class cl = DataManager.classifiersManager.getClassifierClass(info[selected].name);
				StandaloneSavableConfiguration ssc = new 
					StandaloneSavableConfiguration(cl);
				chosen.add(new ClassifierInstanceInfo(info[selected].getNextName(), info[selected].flags, ssc, cl));
				chosenList.setListData(chosen.toArray());
			}
		}
		else if (button == removeButton){
			/* Usuwamy klasyfikator */
			int selected = chosenList.getSelectedIndex();
			if (selected >= 0) {
				chosen.remove(selected);
				chosenList.setListData(chosen.toArray());
			}
		} else if (button == optionsButton) {
			/* Ustalamy opcje klasyfikatora */
			int selected = chosenList.getSelectedIndex();
			if (selected >= 0) {
				ClassifierInstanceInfo inf = (ClassifierInstanceInfo) chosen.get(selected);
				new ClassifierOptions(inf.config, frame, inf.name);
			}
			 
		}
		
	}

}
