/*
 * Created on 2005-03-22
 *
 */
package vr.visual;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import vr.system.options.StandaloneSavableConfiguration;

/**
 * Klasa odpowiedzialna za okno ustawiania opcji dla podanej klasy lub podanych jako parametr opcji. 
 * Zmienione opcje zapisywane sa do obiektu StandaloneSavableConfiguration, ktory mozna od klasy
 * pobrac.
 * 
 * @author Marcin Staszczyk
 */
public class ClassifierOptions extends JFrame implements ActionListener {
	
	/** Klasa przechowujaca opcje. */
	private StandaloneSavableConfiguration conf;
	/** Mapa przechwujaca pod nazwami opcji interfejsowe pola tekstowe zawierajace warosci. */
	private Map map = new HashMap();
	
	/** Panel na ktorym wszystko wyswietlamy. */
	JPanel panel;
	
	/**
	 * Przyciski widoczne na panelu
	 */
	private JButton okButton;
	private JButton anulujButton;
	private JButton zapiszButton;
	private JButton nowaButton;
	
	/**
	 * Konstruktor klasy czytajacy opcje z domyslnego pliku dla podanej klasy.
	 * 
	 * @param configurableClass Klasa ktorej opcje ustawiamy.
	 * @param parent			
	 */
	public ClassifierOptions(Class configurableClass, Component parent) {
		conf = new StandaloneSavableConfiguration(configurableClass);
		
		createWindow();
		setLocationRelativeTo(parent);
	}
	
	/**
	 * Konstruktor klasy czytujacy opcje z klasy StandaloneSavableConfiguration, a zmienione opcje sa do niej zapisywane.
	 * 
	 * @param config	Klasa z ktorej pobieramy opcje.
	 * @param parent			
	 * @param name		Nazwa klasy dla ktorej opcje ustawiamy.
	 */
	public ClassifierOptions(StandaloneSavableConfiguration config, Component parent, String name) {
		super("Opcje dla "+name);
		conf = config;
		
		createWindow();
		setLocationRelativeTo(parent);
	}
	
	/**
	 * Tworzy okno umozliwiajace zmiane opcji.
	 *
	 */
	private void createWindow()
	{
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		JPanel mainPanel = new JPanel(new GridBagLayout());
		
		panel = new JPanel(new GridBagLayout());
		JScrollPane scrollPanel = new JScrollPane(panel,
				                                  JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                                  JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		GridBagConstraints c1 = new GridBagConstraints();
        c1.gridwidth = GridBagConstraints.NONE;
        c1.fill = GridBagConstraints.HORIZONTAL;
        c1.weightx = 1.0;
		
		Enumeration itr = conf.getPropertiesNames();
		while (itr.hasMoreElements())
		{
			String name = (String)itr.nextElement();
			String val = conf.getProperty(name);
			
			JPanel localPanel = new JPanel(new GridBagLayout());
			JLabel label = new JLabel(name + " =  ");
			JTextField textField = new JTextField(50);
			textField.setText(val);
			
			map.put(name,textField);
			
			localPanel.add(label,c1);
			localPanel.add(textField,c1);
			localPanel.setBorder(BorderFactory.createTitledBorder(""));
			panel.add(localPanel,c1);
		}
		
        c1.weighty = 1.0;
		c1.gridwidth = GridBagConstraints.REMAINDER;
		c1.fill = GridBagConstraints.BOTH;
		mainPanel.add(scrollPanel,c1);
		
		JPanel przycPanel = new JPanel(new GridBagLayout());
		
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		anulujButton = new JButton("Anuluj");
		anulujButton.addActionListener(this);
		zapiszButton = new JButton("Zapisz");
		zapiszButton.addActionListener(this);
		nowaButton = new JButton("Nowa pozycja");
		nowaButton.addActionListener(this);
		
		przycPanel.add(okButton);
		przycPanel.add(anulujButton);
		przycPanel.add(zapiszButton);
		przycPanel.add(nowaButton);
		mainPanel.add(przycPanel);
		
		mainPanel.setOpaque(true);
		
		setContentPane(mainPanel);
		pack();
		setVisible(true);
	}

	/**
	 * Obsluga akcji uzytkownika.
	 */
	public void actionPerformed(ActionEvent arg0) {
		if ((arg0.getSource() == zapiszButton) || (arg0.getSource() == okButton))
		{
			Iterator itr = map.keySet().iterator();
			while (itr.hasNext())
			{
				String name = (String)itr.next();
				JTextField textField = (JTextField)map.get(name);
				conf.setProperty(name, textField.getText());
			}
		}
		if ((arg0.getSource() == anulujButton) || (arg0.getSource() == okButton))
		{
			this.setVisible(false);
			this.dispose();
		}
		
		
		if (arg0.getSource() == nowaButton)
		{
			JFrame frame = new JFrame();
			
			String s = (String)JOptionPane.showInputDialog(
                    frame,
                    "Podaj nazwe nowej opcji",
                    "Nowa opcja",
                    JOptionPane.PLAIN_MESSAGE);
			
			if (s == null)
			{
				frame.dispose();
				return;
			}
				
			
			if (s.length() == 0)
			{
				JOptionPane.showMessageDialog(frame,
                        "Nazwa opcji nie moze byc pusta",
                        "BLAD",
                        JOptionPane.ERROR_MESSAGE);
				frame.dispose();
				return;
			}
			
			if (map.containsKey(s))
			{
				JOptionPane.showMessageDialog(frame,
                        "Podana opcja juz istnieje",
                        "BLAD",
                        JOptionPane.ERROR_MESSAGE);
				frame.dispose();
				return;
			}
			
			frame.dispose();
			
			GridBagConstraints c1 = new GridBagConstraints();
	        c1.gridwidth = GridBagConstraints.NONE;
	        c1.fill = GridBagConstraints.HORIZONTAL;
	        c1.weightx = 1.0;
			
			JPanel localPanel = new JPanel(new GridBagLayout());
			JLabel label = new JLabel(s + " =  ");
			JTextField textField = new JTextField(50);
			textField.setText("");
			
			map.put(s,textField);
			
			localPanel.add(label,c1);
			localPanel.add(textField,c1);
			localPanel.setBorder(BorderFactory.createTitledBorder(""));
			panel.add(localPanel,c1);
			pack();
		}	
	}
}
