/*
 * Created on 2005-03-20
 *
 */
package vr.visual;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import rseslib.system.Report;
import vr.system.classifiersManagement.ClassFileFilter;
import vr.system.classifiersManagement.ClassifierRegistrator;

/**
 * Klasa odpowiedzialna za interfejsowa czesc rejestracji klasyfikatorow.
 * Umozliwia wybranie klasyfikatora do rejestracji z katalogow projektu.
 *
 * @author Marcin Staszczyk
 */
public class ClassifierRegistratorThread extends Thread {

	/** Obiekt odpowiedzialny za wybor klasyfikatora. */
	static private JFileChooser fc = new JFileChooser();

	/** Inicjalizacja powyzszego obiektu. */
	static {
		fc.setMultiSelectionEnabled(false);
        fc.setCurrentDirectory(new File("."));
        fc.setAcceptAllFileFilterUsed(false);
        fc.addChoosableFileFilter(new ClassFileFilter());
	}

	/**
	 * Metoda uruchamiajaca wizualna czesc rejestrcji nowego klasyfikatora.
	 */
	public void run()
	{
		int returnVal = fc.showOpenDialog(null);
        if (returnVal != JFileChooser.APPROVE_OPTION)
        {
        	return;
        }

        File file = fc.getSelectedFile();
        String runDirectory = new File(".").getPath();

        ClassifierRegistrator reg = new ClassifierRegistrator();

        File file1, file2;
    	file1 = file;
    	while (true)
    	{
    		file2 = file1.getParentFile();
    		if ((file2 == null) || (!file1.getPath().startsWith(file2.getPath())))
    			break;
    		file1 = file2;
    	}

    	String path = makeJavaPath(file.getPath().substring(file1.getPath().length()));
    	while(true)
    	{
    		try {
				Class.forName(path);
				break;
			} catch (ClassNotFoundException e1) {}
			catch (NoClassDefFoundError e2) {}
			//catch (Throwable e3) { Report.exception( new RuntimeException("Wystapil nieprzewidziany blad procesu wyszukiwania scierzki w rejestracji klasyfikatora\n" + e3.getMessage())); }

			int sub = path.indexOf('.');
			if ((sub < 0) || (path.length() == 0) || (sub > path.length()))
    		{
    			JOptionPane.showMessageDialog(new JFrame(), "Niestety nie udalo sie dopasowac pliku do zadnej sciezki z CLASSPATH", "Blad", JOptionPane.ERROR_MESSAGE);
    			return;
    		}
    		path = path.substring(sub + 1);
    	}

    	try {
			if (reg.classifier(path, 32))
			{
				Object[] opt = {"Tak", "Nie"};

				int n = JOptionPane.showOptionDialog(
						new JFrame(),
						"Klasyfikator byl juz wczesniej rejestrowany.\n" +
						"Czy chcesz nadpisac dane poprzedniej rejestracji?",
						"Rejestracja Klasyfikatora",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.QUESTION_MESSAGE,
						null,
						opt,
						opt[1]);

				if (n == JOptionPane.NO_OPTION)
				{
					return;
				}
			}
		} catch (Exception e) {
			Report.exception(e);
			return;
		}

        try {
			reg.noProperties();
		} catch (RuntimeException e) {
			Report.exception(e);
			return;
		}

		try {
			reg.addClassifier();
		} catch (RuntimeException e) {
			Report.exception(e);
			return;
		}
	}

	/**
	 * Tworzy z obiektu typu file scierzke wzgledna od katalogu projektu
	 * do pliku z klasa w postaci java-owej.
	 * Odcina pierwsza czesc scierzki bedaca scierzka do aktualnego katalogu.
	 *
	 * @param classifier	plik z klasa
	 * @return				scierzka wzgledna od katalogu projektu
	 * do pliku z klasa w postaci java-owej
	 */
	private String makeJavaPath(String classifier)
	{
		String pom = classifier.replace(File.separatorChar,'.');
		return pom.substring(0,pom.length()-6);
	}
}
