/*
 * @(#)AbstractReport.java        0.8 05/03/04
 *
 * Copyright (c) 2002 Warsaw University
 */

package vr.visual;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Rectangle;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Java Swing panel sluzacy do wyswietlania komunikatow LOG
 * Zawira pole komunikatow o bledach, komunikatow debagowyvh
 * i komunikatow informacyjnych
 *
 * @version 0.8 5 Mar 2005
 * @author Marcin Staszczyk
 *
 */
public class LogPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	private final static String newline = "\n";


	protected String headerFile;
	protected String trainingFile;
	protected String testFile;

	protected boolean headerModified;
	protected boolean trainingModified;
	protected boolean testModified;

	protected JLabel headerLabel;
	protected JLabel trainingLabel;
	protected JLabel testLabel;

	protected JTextArea errorArea;
	protected JTextArea debugArea;
	protected JTextArea displayArea;
	protected JScrollPane errorPanel;
	protected JScrollPane debugPanel;
	protected JScrollPane displayPanel;
	protected JLabel errorLabel;
	protected JLabel debugLabel;
	protected JLabel displayLabel;

	/**
	 * Konstruktor klasy tworzacy wszystkie potrzebne klasie do dzialania pola.
	 * Definjuje uklad elementow na panelu.
	 *
	 */
	public LogPanel() {
        super(new GridBagLayout());

        setBorder(BorderFactory.createEmptyBorder(20,20,20,20));

        GridBagConstraints c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 1.0;
        c.weighty = 1.0;


        add(headerLabel = new JLabel(), c);
        add(trainingLabel = new JLabel(), c);
        add(testLabel = new JLabel(), c);
        refreshFiles();


        errorArea = new JTextArea(4,40);
        errorArea.setEditable(false);
        errorPanel = new JScrollPane(errorArea,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        errorPanel.setAutoscrolls(true);

        debugArea = new JTextArea(4,40);
        debugArea.setEditable(false);
        debugArea.scrollRectToVisible(new Rectangle(4,40));
        debugPanel = new JScrollPane(debugArea,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        

        displayArea = new JTextArea(4,40);
        displayArea.setEditable(false);
        displayPanel = new JScrollPane(displayArea,
                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);


        errorLabel = new JLabel("Bledy");
        debugLabel = new JLabel("Debug");
        displayLabel = new JLabel("Info");



        add(errorLabel);
        add(errorPanel, c);
        add(debugLabel);
        add(debugPanel, c);
        add(displayLabel);
        add(displayPanel, c);
	}

	/**
	 * Odswieza wyswietlane informacje o wczytanych przez VR plikach z danymi: naglowkiem, danymi treningowymi i testowymi
	 *
	 */
	private void refreshFiles()
	{
		headerLabel.setText("            Naglowek danych: " + (headerFile == null ? "brak" : headerFile) + (headerModified ? " *" : ""));
		trainingLabel.setText("            Dane treningowe: " + (trainingFile == null ? "brak" : trainingFile) + (trainingModified ? " *" : ""));
		testLabel.setText("            Dane testowe: " + (testFile == null ? "brak" : testFile) + (testModified ? " *" : ""));
	}


	/**
	 * wyswietla komunikat na ekran w miejscu przeznaczonym na komunikaty o bledach
	 *
	 * @param message komunikat o bledzie
	 */
	public void error(String message)
	{
		errorArea.append(message);
	}

	/**
	 * wyswietla komunikat na ekran w miejscu przeznaczonym na komunikaty debagowe.
	 * Zakoncza go znakiem nowej linii
	 *
	 * @param message komunikat debagowy
	 */
	public void debug(String message)
	{
		debugArea.append(message);
	}

	/**
	 * przechodzi do nowej linii na ekrane w miejscu przeznaczonym na komunikaty debagowe.
	 *
	 */
	public void debugnl()
	{
		debugArea.append(newline);
	}

	/**
	 * wyswietla komunikat zwiazany z obiektem na ekran w miejscu przeznaczonym na komunikaty informacyjne.
	 *
	 * @param obj Obiek od ktorego pobierzemy komunikat.
	 */
	public void display(Object obj)
	{
		displayArea.append(obj.toString());
	}

	/**
	 * wyswietla znak nowej linii.
	 */
	public void displaynl()
	{
		displayArea.append(newline);
	}

	/**
     * Otwiera lub zamyka okno komunikatow o bledach
     *
     * @param open flaga ustalajaca, czy okno ma byc otwarte
     */
	public void setOpenErrorWindow(boolean open)
    {
		errorPanel.setVisible(open);
		errorLabel.setVisible(open);
    }

	/**
     * Otwiera lub zamyka okno komunikatow debagowych
     *
     * @param open flaga ustalajaca, czy okno ma byc otwarte
     */
    public void setOpenDebugWindow(boolean open)
    {
    	debugPanel.setVisible(open);
    	debugLabel.setVisible(open);
    }

    /**
     * Otwiera lub zamyka okno komunikatow informacyjnych
     *
     * @param open flaga ustalajaca, czy okno ma byc otwarte
     */
    public void setOpenDisplayWindow(boolean open)
    {
    	displayPanel.setVisible(open);
    	displayLabel.setVisible(open);
    }

    /**
     * Ustawia informacje o wczytanym obecnie pliku z naglowkiem.
     * Jesli f == null lub dlugosci 0 to traktowane jest ze zaden
     * plik nie jest wczytany i wtedy odpowiedni komunikat
     * zostanie wyswietlony.
     *
     * @param f Informacje o wczytanym pliku.
     */
    public void setHeaderFile(String f)
    {
    	headerFile = f;
    	refreshFiles();
    }
    /**
     * Ustawia informacje o wczytanym obecnie pliku z danymi treningowymi.
     * Jesli f == null lub dlugosci 0 to traktowane jest ze zaden
     * plik nie jest wczytany i wtedy odpowiedni komunikat
     * zostanie wyswietlony.
     *
     * @param f Informacje o wczytanym pliku.
     */
    public void setTrainingFile(String f)
    {
    	trainingFile = f;
    	refreshFiles();
    }
    /**
     * Ustawia informacje o wczytanym obecnie pliku z danymi testowymi.
     * Jesli f == null lub dlugosci 0 to traktowane jest ze zaden
     * plik nie jest wczytany i wtedy odpowiedni komunikat
     * zostanie wyswietlony.
     *
     * @param f Informacje o wczytanym pliku.
     */
    public void setTestFile(String f)
    {
    	testFile = f;
    	refreshFiles();
    }
    /**
     * Ustawia czy naglowek byl modyfikowany
     */
    public void setHeaderModified(boolean b)
    {
    	headerModified = b;
    	refreshFiles();
    }
    /**
     * Ustawia czy dane treningowe byly modyfikowany
     */
    public void setTrainingModified(boolean b)
    {
    	trainingModified = b;
    	refreshFiles();
    }
    /**
     * Ustawia czy dane testowe byly modyfikowany
     */
    public void setTestModified(boolean b)
    {
    	testModified = b;
    	refreshFiles();
    }

}
