/*
 * Created on 2005-04-08
 *
 */
package vr.visual;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import rseslib.system.Report;
import vr.system.DataManager;
import vr.system.options.StandaloneSavableConfiguration;

/**
 * Klasa odpowiedzialna za ustawianie opcji programu VR.
 *
 * @author Marcin Staszczyk
 */
public class VROptions extends JFrame implements ActionListener {

	/**
	 * Stale do pobierania opcji z pliku konfiguracyjnego
	 */
	private static final String displayDebugYes = "displayDebugYes";
	private static final String displayDebugNo = "displayDebugNo";
	private static final String displayDisplayYes = "displayDisplayYes";
	private static final String displayDisplayNo = "displayDisplayNo";
	private static final String windowErrorOn = "windowErrorOn";
	private static final String windowErrorOff = "windowErrorOff";
	private static final String windowDebugOn = "windowDebugOn";
	private static final String windowDebugOff = "windowDebugOff";
	private static final String windowDisplayOn = "windowDisplayOn";
	private static final String windowDisplayOff = "windowDisplayOff";

	public static final String displayDebug = "displayDebug";
	public static final String displayDisplay = "displayDisplay";
	public static final String windowError = "windowError";
	public static final String windowDebug = "windowDebug";
	public static final String windowDisplay = "windowDisplay";

	public static final String debugFileProp = "errorFileProp";
	public static final String displayFileProp = "ddFileProp";

	/**
	 * aktualne ustawienia opcji
	 */
	private boolean dDebug;
	private boolean dDisplay;
	private boolean wError;
	private boolean wDebug;
	private boolean wDisplay;

	/**
	 * grupy przyciskow widocznych w oknie
	 */
	private ButtonGroup gDDeb;
	private ButtonGroup gDDis;
	private ButtonGroup gWErr;
	private ButtonGroup gWDeb;
	private ButtonGroup gWDis;

	/**
	 * pola tekstowe zawierajace tekstowe opcje
	 */
	private JTextField errorFileField;
	private JTextField ddFileField;

	/**
	 * aktualne ustawienia opcji
	 */
	private String debugFile;
	private String displayFile;

	/**
	 * przyciski sterujace z interfejsu
	 */
	private JButton okButton;
	private JButton anulujButton;
	private JButton zapiszButton;

	/** Panel na ktorym pokazujemy komponenty. */
	JPanel mainPanel = new JPanel(new GridBagLayout());

	/** Klasa odpowiedzialna z zapisywanie opcji do pliku. */
	private StandaloneSavableConfiguration conf;

	/**
	 * Konstruktor tworzacy okno
	 *
	 */
	public VROptions() {
		super("Opcje");
		conf = new StandaloneSavableConfiguration(VROptions.class);

		dDebug = conf.getBoolProperty(displayDebug);
		dDisplay = conf.getBoolProperty(displayDisplay);
		wError = conf.getBoolProperty(windowError);
		wDebug = conf.getBoolProperty(windowDebug);
		wDisplay = conf.getBoolProperty(windowDisplay);

		debugFile = conf.getProperty(debugFileProp);
		displayFile = conf.getProperty(displayFileProp);


		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		JPanel panel;
		JRadioButton[] radioButtons = new JRadioButton[2];
		ButtonGroup group;
		JLabel label;

		GridBagConstraints c1 = new GridBagConstraints();
        c1.gridwidth = GridBagConstraints.NONE/*REMAINDER*/;
        c1.fill = GridBagConstraints.HORIZONTAL;

        panel = new JPanel(new GridBagLayout());



		label = new JLabel("VRWindow komunikatow error:");

		radioButtons[0] = new JRadioButton("Wlaczone");
        radioButtons[0].setActionCommand(windowErrorOn);

        radioButtons[1] = new JRadioButton("Wylaczone");
        radioButtons[1].setActionCommand(windowErrorOff);

        group = new ButtonGroup();

        group.add(radioButtons[0]);
        group.add(radioButtons[1]);
        radioButtons[(wError ? 0 : 1)].setSelected(true);

        gWErr = group;

        panel.add(label,c1);
        panel.add(radioButtons[0],c1);
        panel.add(radioButtons[1],c1);

        mainPanel.add(panel,c1);



		label = new JLabel("Wypisywanie komunikatow debagowych:");

		radioButtons[0] = new JRadioButton("TAK");
        radioButtons[0].setActionCommand(displayDebugYes);

        radioButtons[1] = new JRadioButton("NIE");
        radioButtons[1].setActionCommand(displayDebugNo);

        group = new ButtonGroup();

        group.add(radioButtons[0]);
        group.add(radioButtons[1]);
        radioButtons[(dDebug ? 0 : 1)].setSelected(true);

        gDDeb = group;

        panel.add(label,c1);
        panel.add(radioButtons[0],c1);
        panel.add(radioButtons[1],c1);



        label = new JLabel("VRWindow komunikatow debagowych:");

		radioButtons[0] = new JRadioButton("Wlaczone");
        radioButtons[0].setActionCommand(windowDebugOn);

        radioButtons[1] = new JRadioButton("Wylaczone");
        radioButtons[1].setActionCommand(windowDebugOff);

        group = new ButtonGroup();

        group.add(radioButtons[0]);
        group.add(radioButtons[1]);
        radioButtons[(wDebug ? 0 : 1)].setSelected(true);

        gWDeb = group;

        panel.add(label,c1);
        panel.add(radioButtons[0],c1);
        panel.add(radioButtons[1],c1);

        mainPanel.add(panel,c1);



        label = new JLabel("Wypisywanie komunikatow informacyjnych:");

		radioButtons[0] = new JRadioButton("TAK");
        radioButtons[0].setActionCommand(displayDisplayYes);

        radioButtons[1] = new JRadioButton("NIE");
        radioButtons[1].setActionCommand(displayDisplayNo);

        group = new ButtonGroup();

        group.add(radioButtons[0]);
        group.add(radioButtons[1]);
        radioButtons[(dDisplay ? 0 : 1)].setSelected(true);

        gDDis = group;

        panel.add(label,c1);
        panel.add(radioButtons[0],c1);
        panel.add(radioButtons[1],c1);



        label = new JLabel("VRWindow komunikatow informacyjnych:");

		radioButtons[0] = new JRadioButton("Wlaczone");
        radioButtons[0].setActionCommand(windowDisplayOn);

        radioButtons[1] = new JRadioButton("Wylaczone");
        radioButtons[1].setActionCommand(windowDisplayOff);

        group = new ButtonGroup();

        group.add(radioButtons[0]);
        group.add(radioButtons[1]);
        radioButtons[(wDisplay ? 0 : 1)].setSelected(true);

        gWDis = group;

        panel.add(label,c1);
        panel.add(radioButtons[0],c1);
        panel.add(radioButtons[1],c1);



        label = new JLabel("Plik do komunikatow o bledach i debugowych:");
        errorFileField = new JTextField(40);
        errorFileField.setText(debugFile);
        panel.add(label,c1);
        panel.add(errorFileField,c1);

        label = new JLabel("Plik do komunikatow informacyjnych:");
        ddFileField = new JTextField(40);
        ddFileField.setText(displayFile);
        panel.add(label,c1);
        panel.add(ddFileField,c1);


        mainPanel.add(panel,c1);


        JPanel przycPanel = new JPanel(new GridBagLayout());

		okButton = new JButton("OK");
		okButton.addActionListener(this);
		anulujButton = new JButton("Anuluj");
		anulujButton.addActionListener(this);
		zapiszButton = new JButton("Zapisz");
		zapiszButton.addActionListener(this);

		przycPanel.add(okButton);
		przycPanel.add(anulujButton);
		przycPanel.add(zapiszButton);
		mainPanel.add(przycPanel);


        mainPanel.setOpaque(true);

        setContentPane(mainPanel);
        pack();
        setLocationRelativeTo(VRWindow.mainWindow);
        setVisible(true);
	}

	/**
	 * Obsluga akcji uzytkonika
	 */
	public void actionPerformed(ActionEvent arg0) {
		if ((arg0.getSource() == zapiszButton) || (arg0.getSource() == okButton))
		{
			String command = gDDeb.getSelection().getActionCommand();
			dDebug = command == displayDebugYes;
			Report.setDebugDisplay(dDebug);
			conf.setProperty(displayDebug, (dDebug ? "TRUE" : "FALSE"));

			command = gDDis.getSelection().getActionCommand();
			dDisplay = command == displayDisplayYes;
			Report.setInfoDisplay(dDisplay);
			conf.setProperty(displayDisplay, (dDisplay ? "TRUE" : "FALSE"));

			command = gWErr.getSelection().getActionCommand();
			wError = command == windowErrorOn;
			DataManager.panel.setOpenErrorWindow(wError);
			conf.setProperty(windowError, (wError ? "TRUE" : "FALSE"));

			command = gWDeb.getSelection().getActionCommand();
			wDebug = command == windowDebugOn;
			DataManager.panel.setOpenDebugWindow(wDebug);
			conf.setProperty(windowDebug, (wDebug ? "TRUE" : "FALSE"));

			command = gWDis.getSelection().getActionCommand();
			wDisplay = command == windowDisplayOn;
			DataManager.panel.setOpenDisplayWindow(wDisplay);
			conf.setProperty(windowDisplay, (wDisplay ? "TRUE" : "FALSE"));

			debugFile = errorFileField.getText();
			if (debugFile.length() > 0)
			{
				File tmp = new File(debugFile);
				try {
					if (((tmp.exists()) && (tmp.isFile())) || (tmp.createNewFile()))
						conf.setProperty(debugFileProp, debugFile);
				} catch (IOException e) {
					JFrame frame = new JFrame();
					JOptionPane.showMessageDialog(frame,
	                        "Podany plik do zapisu bledow nie"
							+ "\nistnieje i nie moge go utworzyc."
							+ "\nPlik nie zostanie zapisany.",
	                        "BLAD",
	                        JOptionPane.ERROR_MESSAGE);
					frame.dispose();
				}
			}


			displayFile = ddFileField.getText();
			if (displayFile.length() > 0)
			{
				File tmp = new File(displayFile);
				try {
					if (((tmp.exists()) && (tmp.isFile())) || (tmp.createNewFile()))
						conf.setProperty(displayFileProp, displayFile);
				} catch (IOException e) {
					JFrame frame = new JFrame();
					JOptionPane.showMessageDialog(frame,
	                        "Podany plik do zapisu komunikatow debagowych i informacyjnych"
							+ "\nnie istnieje i nie moge go utworzyc."
							+ "\nPlik nie zostanie zapisany.",
	                        "BLAD",
	                        JOptionPane.ERROR_MESSAGE);
					frame.dispose();
				}
			}


			conf.writeProperties();
		}
		if ((arg0.getSource() == anulujButton) || (arg0.getSource() == okButton))
		{
			this.setVisible(false);
			this.dispose();
		}


	}
}
