/*
 * Created on Apr 9, 2005
 *
 */
package vr.visual;


import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import vr.data.LastClassificationInfo;
import vr.system.DataManager;
import vr.visual.data.DataEditorPanel;
import vr.visual.data.HeaderEditorPanel;
import vr.visual.dataIO.HeaderReader;
import vr.visual.dataIO.HeaderWriter;
import vr.visual.dataIO.TestDataReader;
import vr.visual.dataIO.TestDataWriter;
import vr.visual.dataIO.TrainDataReader;
import vr.visual.dataIO.TrainDataWriter;
import vr.visual.graph.GraphSetupPanel;
import vr.visual.results.ClassificationResultsPanel;


/**
 *
 * Klasa tworzaca glowny interfejs programu. Odpowiedzialna za utworzenie glownego okna programu i paska menu.
 * 
 * @author Tomek Romanczuk, Michal Manski
 */
public class VRWindow implements ActionListener {
	
	/** Okno "VReslib info" */
	private JFrame oknoX;
	/** Glowny panel, do ktorego wkladane sa poszczegolne panele takie jak panel z danymi czy panel do wyboru grafu. */
	static JPanel mainPanel;       
	/** Glowne okno programu */
	public static JFrame mainWindow;
	
	private static final String DANE_TESTOWE = "Dane Testowe";
	private static final String DANE_TRENINGOWE = "Dane Treningowe";
	private static final String NAGLOWEK_DANYCH = "Naglowek danych";
	private static final String WYNIKI_KLASYFIKACJI = "Wyniki ostatniej klasyfikacji";
	
	private static final String STWORZ_WYKRES = "Stworz wykres";
	
	private static final String WCZYTAJ_PLIK_NAGLOWKOWY = "Wczytaj plik naglowkowy";
	private static final String ZAPISZ_PLIK_NAGLOWKOWY = "Zapisz plik naglowkowy";
	private static final String WCZYTAJ_PLIK_TRENINGOWY = "Wczytaj dane treningowe";
	private static final String ZAPISZ_PLIK_TRENINGOWY = "Zapisz dane treningowe";
	private static final String WCZYTAJ_PLIK_TESTOWY = "Wczytaj dane testowe";
	private static final String ZAPISZ_PLIK_TESTOWY = "Zapisz dane testowe";
	private static final String KONIEC = "Koniec";
	
	private static final String WYBOR_KLASYFIKATORA = "Wybor klasyfikatora";
	private static final String REJESTRACJA_KLASYFIKATORA = "Rejestracja klasyfikatora";
	private static final String KLASYFIKUJ = "Klasyfikuj";
	private static final String ZAPISZ_RAPORT = "Zapisz raport";
	
	private static final String VRSESLIB_INFO = "VRseslib - info";
	
	private static final String OPCJE = "Opcje";
	
	
	/**
	 * Klasa odpowiedzialna za zamkniecie okna programu.
	 * 
	 * @author Tomek Romanczuk
	 *
	 */
	private class End implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			oknoX.dispose();
		}
	}
	
	/**
	 * tworzenie paska opcji w menu
	 * 
	 * @return pasek opcji
	 */
	public JMenuBar createMenuBar()
	{
		JMenuBar pasek;
		JMenu menu;
		JMenuItem elem;
		
		pasek = new JMenuBar();
		/* tworzenie kolejnych menu */
		menu = new JMenu("Plik");
		elem = new JMenuItem(WCZYTAJ_PLIK_NAGLOWKOWY);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(ZAPISZ_PLIK_NAGLOWKOWY);
		elem.addActionListener(this);
		menu.add(elem);
		menu.addSeparator();
		elem = new JMenuItem(WCZYTAJ_PLIK_TRENINGOWY);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(ZAPISZ_PLIK_TRENINGOWY);
		elem.addActionListener(this);
		menu.add(elem);
		menu.addSeparator();
		elem = new JMenuItem(WCZYTAJ_PLIK_TESTOWY);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(ZAPISZ_PLIK_TESTOWY);
		elem.addActionListener(this);
		menu.add(elem);
		menu.addSeparator();
		elem = new JMenuItem(KONIEC);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		
		/*kolejne menu - WYKRES */
		menu = new JMenu("Wykres");
		elem = new JMenuItem(STWORZ_WYKRES);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		
		menu = new JMenu("Dane");
		elem = new JMenuItem(NAGLOWEK_DANYCH);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(DANE_TRENINGOWE);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(DANE_TESTOWE);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		
		menu = new JMenu("Klasyfikacja");
		elem = new JMenuItem(WYBOR_KLASYFIKATORA);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(REJESTRACJA_KLASYFIKATORA);
		elem.addActionListener(this);
		menu.add(elem);
		elem = new JMenuItem(KLASYFIKUJ);
		elem.addActionListener(this);
		menu.add(elem);
		menu.addSeparator();
		elem = new JMenuItem(WYNIKI_KLASYFIKACJI);
		elem.addActionListener(this);
		menu.add(elem);
		menu.addSeparator();
		elem = new JMenuItem(ZAPISZ_RAPORT);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		
		menu = new JMenu("Opcje");
		elem = new JMenuItem(OPCJE);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		
		menu = new JMenu("Pomoc");
		elem = new JMenuItem(VRSESLIB_INFO);
		elem.addActionListener(this);
		menu.add(elem);
		pasek.add(menu);
		return pasek;
	}

	/**
	 * zakonczenie dzialania programu
	 *
	 */
	public void closeProgram()
	{
		System.exit(0);	
	}

	
	/**
	 * Stworzenie okna informacji o programie.
	 * @return okno
	 */
	public JFrame infoWindow()
	{
		String napis;
		JPanel panel = new JPanel(new GridLayout(11,11));
		JPanel panel1, panel2;
		JButton koniec;
		JLabel et;
		
		oknoX = new JFrame("VRselib-info");
		
        
		napis = "Autorzy: ";
		et = new JLabel(napis);
		panel.add(et);
		napis = "        Damian Manski";
		et = new JLabel(napis);
		panel.add(et);
		napis = "        Michal Manski";
		et = new JLabel(napis);
		panel.add(et);
		napis = "        Tomasz Romanczuk";
		et = new JLabel(napis);
		panel.add(et);
		napis = "        Marcin Staszczyk";
		et = new JLabel(napis);
		panel.add(et);
		napis = "";
		et = new JLabel(napis);
		panel.add(et);
		napis = "Czas powstania programu:  luty - czerwiec 2005";
		et = new JLabel(napis);
		panel.add(et);
		napis = "";
		et = new JLabel(napis);
		panel.add(et);
		napis = "Zyczymy milego korzystania z naszego programu";
		et = new JLabel(napis);
		panel.add(et);
		napis = "";
		et = new JLabel(napis);
		panel.add(et);
		
		koniec = new JButton("Koniec");
		koniec.addActionListener(new End());
		panel1 = new JPanel();
		panel2 = new JPanel();
		
		panel.add(panel1);
		panel1.add(panel2);
		panel1.add(koniec);
		
		oknoX.setContentPane(panel);
		panel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		return oknoX;
	}

	/**
	 * Wyczyszczenie dotychczasowego glownego panelu i wstawienie do niego panelu "panel".
	 * 
	 * @param panel 	nowy panel
	 */
	public static void setNewContent(JPanel panel) {
		mainPanel.removeAll();
		mainPanel.add(panel, BorderLayout.CENTER); 
		mainPanel.updateUI();
	}
	
	/** 
	 * Metoda "startowa". Tworzy glowne okno programu i pasek menu.
	 * Nie zwraca uwagi na argumenty wywolania.
	 * 
	 * @param args argumenty programu
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		/* tworzenie okna */
		JFrame okno = new JFrame("VRseslib");
		okno.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		
		/* utworzenie paska zadan */
		VRWindow pasek;
		pasek = new VRWindow();
		okno.setJMenuBar(pasek.createMenuBar());
		
		JPanel panel = new JPanel();
		
		JPanel labelPanel = new JPanel();
		labelPanel.add(new JLabel("Witamy w programie VRseslib!!!"));
		panel.setLayout(new BorderLayout());
		panel.add(labelPanel, BorderLayout.CENTER);
		
		JPanel framePanel = new JPanel();
		
		mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(panel, BorderLayout.CENTER);
		framePanel.setLayout(new BorderLayout());
		framePanel.add(mainPanel, BorderLayout.CENTER);
		framePanel.add(DataManager.panel, BorderLayout.SOUTH);
		

		
		okno.setContentPane(framePanel);
		/* wyswietlenie okna */
		okno.setSize(800,600);
		okno.setExtendedState(JFrame.MAXIMIZED_BOTH);
		okno.setVisible(true);
		
		mainWindow = okno;
	}

	/**
	 * Wypisuje na ekranie podana informacje w oknie dialogowym.
	 * 
	 * @param text tekst do wypisania
	 */
	public void writeInfo(String text) {
		JOptionPane.showMessageDialog(mainWindow, text, "Informacja", JOptionPane.INFORMATION_MESSAGE);
	}
	
	/**
	 * Obsluga akcji podjetych przez uzytkownika.
	 */
	public void actionPerformed(ActionEvent arg0) {
		JMenuItem opcja = (JMenuItem)(arg0.getSource());
		String napis = opcja.getText();
		
		if (napis == WCZYTAJ_PLIK_NAGLOWKOWY)
		{
			new HeaderReader();
		}
			
		if (napis == ZAPISZ_PLIK_NAGLOWKOWY) 
			if (DataManager.dataHeader != null)
			{
				new HeaderWriter();
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nNaglowek danych nie jest wczytany.");
		
		if (napis == WCZYTAJ_PLIK_TRENINGOWY)
			if (DataManager.dataHeader != null)
			{
				new TrainDataReader();
			}
			else
				writeInfo("Przed wczytaniem danych treningowych\nnalezy wczytac naglowek danych.");
		
		if (napis == ZAPISZ_PLIK_TRENINGOWY)
			if (DataManager.trainData != null)
			{
				new TrainDataWriter();
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nDane treningowe nie sa wczytane.");
		
		if (napis == WCZYTAJ_PLIK_TESTOWY)
			if (DataManager.dataHeader != null)
			{
				new TestDataReader();
			}
			else
				writeInfo("Przed wczytaniem danych testowych\nnalezy wczytac naglowek danych.");
		
		if (napis == ZAPISZ_PLIK_TESTOWY) 
			if (DataManager.testData != null)
			{
				new TestDataWriter();
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nDane testowe nie sa wczytane.");
		
		if (napis == NAGLOWEK_DANYCH) {
			setNewContent(new HeaderEditorPanel(DataManager.dataHeader)); 
		}
		
		if (napis == DANE_TESTOWE) 
			if (DataManager.dataHeader != null)
				setNewContent(new DataEditorPanel(DataManager.testData, DataManager.dataHeader, true));
			else
				writeInfo("Panel danych testowych nie moze byc wyswietlony,\ndopoki nie jest wczytany naglowek danych.");
		
		
		if (napis == DANE_TRENINGOWE) 
			if (DataManager.dataHeader != null) {
				setNewContent(new DataEditorPanel(DataManager.trainData, DataManager.dataHeader, false));
			}
			else
				writeInfo("Panel danych treningowych nie moze byc wyswietlony,\ndopoki nie jest wczytany naglowek danych.");
		
		
		if (napis == WYBOR_KLASYFIKATORA)
			if (DataManager.dataHeader != null) 
			{
				new ClassifierChooser();
			}
			else
				writeInfo("Przed wyborem klasyfikatorow nalezy\nwczytac naglowek danych.");
		
		
		if (napis == KLASYFIKUJ) 
			if ((DataManager.trainData != null) && (DataManager.testData != null)) {
				if (DataManager.chosenClassifiers.size() > 0) {
					Thread th = new ClassifyThread();
					th.start();
				}
				else
					writeInfo("Do przeprowadzenia klasyfikacji potrzebne jest\nwybranie conajmniej jednego klasyfikatora.");
			
			}
			else
				writeInfo("Do przeprowadzenia klasyfikacji potrzebne jest\nwczytanie zarowno danych testowych jak i treningowych.");
		
		
		if (napis == WYNIKI_KLASYFIKACJI) 
			if (LastClassificationInfo.getResults() != null) {
				/* Zakladam ze klayfikacja na danych z jednym atrybutem decyzyjnym-nominalnym */
				setNewContent(new ClassificationResultsPanel(LastClassificationInfo.getResults(), LastClassificationInfo.getHeader().nominalDecisionAttribute())); 
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nNie ma w pamieci zadnych wynikow klasyfikacji.");
		
		if (napis == ZAPISZ_RAPORT)
			if (LastClassificationInfo.getResults() != null) {
				LastClassificationInfo.writeResults();
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nNie ma w pamieci zadnych wynikow klasyfikacji.");
			
		
		if (napis == STWORZ_WYKRES) 
			if (DataManager.dataHeader != null) {
				setNewContent(new GraphSetupPanel());
			}
			else
				writeInfo("Operacja nie moze byc wykonana.\nNaglowek danych nie jest wczytany.");
		
		if ( napis == OPCJE)
			new VROptions();
		
		if ( napis == REJESTRACJA_KLASYFIKATORA)
		{
			Thread rej = new ClassifierRegistratorThread();
	        rej.start();
		}
		
		if ( napis == KONIEC)
			closeProgram();

		
		if ( napis == VRSESLIB_INFO ){
			JFrame okienko = infoWindow();
			okienko.setSize(450, 400);
			okienko.setResizable(false);
			okienko.setLocationRelativeTo(mainWindow);
			okienko.setVisible(true);
			
		}
	}

}

