/*
 * Created on 2005-03-19
 *
 */
package vr.visual.data;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;

import rseslib.structure.attribute.Header;
import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.data.ArrayListDoubleDataTable;
import rseslib.structure.data.DoubleData;
import rseslib.structure.data.DoubleDataObject;
import rseslib.structure.data.DoubleDataTable;
import vr.system.DataManager;


/**
 * Panel wyswietlajacy zbior danych testowych lub treningowych.
 * 
 * @author Michal Manski
 *
 */
public class DataPresentationPanel extends PresentationPanel implements ListSelectionListener {
	/** Tabela wyswietlajaca dane. */
	protected JTable table;
	/** Lista wyboru pozwalajaca okreslic, ktore atrybutu obiektow danych maja byc wyswietlane. */
	protected JList list;
	/** Model danych dla tabeli. */
	protected DoubleDataTableModel model;
	/** Naglowek danych pasujacy do wyswietlanych danych. */
	protected Header header;
	/** Okresla czy w tabeli sa dane testowe czy treningowe */
	protected boolean testData;
	
	/**
	 * Konstruktor
	 * 
	 * @param data zbior danych testowych lub treningowych.
	 * @param hdr naglowek danych pasujacy do zbioru danych data.
	 * @param test rowne true wtw data jest.
	 * zbiorem danych testowych.
	 */
	public DataPresentationPanel(DoubleDataTable data, Header hdr, boolean test) {
		
		/* Gdy zbior danych "nie istnieje" to tworze nowy pusty. */
		if (data == null)
			data = new ArrayListDoubleDataTable(hdr);	
		
		Iterator iterator = data.iterator();
		header = data.attributes();
		ArrayList array = new ArrayList();
		ArrayList columns = new ArrayList();
		ArrayList tmpArray;
		
		testData = test;
		
		/* Tworze liste list dla danych tabeli */
		while (iterator.hasNext()) {
			DoubleData obj = (DoubleData) iterator.next();
			tmpArray = new ArrayList();
			for (int i=0; i<header.noOfAttr(); i++) {
				if (header.isNominal(i)) {
					/* Sprawdzam czy jest to missing value */
					if (Double.isNaN(obj.get(i)))
						tmpArray.add(header.missing());
					else
						tmpArray.add(((NominalAttribute) header.attribute(i)).stringValue(obj.get(i)));
				}
				else 
					if (header.isNumeric(i)){
						tmpArray.add(new Double(obj.get(i)));
					}
					else  /* Atrybut tekstowy */
						if (header.attribute(i) instanceof NominalAttribute)
							tmpArray.add(((NominalAttribute) header.attribute(i)).stringValue(obj.get(i)));
						else
							tmpArray.add(new Double(obj.get(i)));
			}
			array.add(tmpArray);
		}
		
		/* Tworze liste dla naglowka tabeli */
		String[] columnNames = new String[header.noOfAttr()];
		for (int i=0; i<header.noOfAttr(); i++) {
			columns.add(header.name(i));
			columnNames[i] = header.name(i);
		}
		
		
		/* Domyslnie ustawiam ze wszystkie kolumny widoczne */
		int[] selected = new int[columnNames.length];
		for (int i=0; i < columnNames.length; i++)
			selected[i] = i;
		
		/* tworzenie tabeli */
		model = new DoubleDataTableModel(array, columns, selected);
		table = new JTable(model) {
			/* Implementowanie "table header tool tips" */ 
            protected JTableHeader createDefaultTableHeader() {
                return new JTableHeader(columnModel) {
                    public String getToolTipText(MouseEvent e) {
                        String tip = null;
                        java.awt.Point p = e.getPoint();
                        int index = columnModel.getColumnIndexAtX(p.x);
                        int realIndex = columnModel.getColumn(index).getModelIndex();
                        if (!header.isInterpretable(realIndex))
                        	return "Atrybut tekstowy";
                        else
                        	if (header.isNominal(realIndex)) {
                        		if (header.isDecision(realIndex))
                        			return "Atrybut decyzyjny, symboliczny";
                        		else
                        			return "Atrybut warunkowy, symboliczny";
                        	}
                        	else 
                        		if (header.isDecision(realIndex)) 
                        			return "Atrybut decyzyjny, numeryczny";
                        		else 
                        			return "Atrybut warunkowy, numeryczny";
                    }
                };
            }
		};
		table.setPreferredScrollableViewportSize(new Dimension(header.noOfAttr()*50, 300));
		
		TableColumn column = null;
		for (int i = 0; i < header.noOfAttr(); i++) {
		    column = table.getColumnModel().getColumn(i);
		    column.setPreferredWidth(50);
		    column.setMinWidth(50);
		}
		
		
		/* Tworzenie listy wyboru */
		list = new JList(columnNames);
		list.addListSelectionListener(this);
		list.setSelectedIndices(selected);
		
		setLayout(new GridBagLayout());
		
		GridBagConstraints c1 = new GridBagConstraints();
		c1.gridwidth = 1;
		c1.fill = GridBagConstraints.BOTH;
		c1.weightx = 8.0;
		c1.weighty = 1.0;
		add(new JScrollPane(table), c1);
		
		JPanel panel = new JPanel(new BorderLayout());
		JPanel labelPanel = new JPanel();
		labelPanel.add(new JLabel("Wyswietlane dane:"));
		panel.add(labelPanel, BorderLayout.NORTH);
		panel.add(new JScrollPane(list), BorderLayout.CENTER);
		
		c1.weightx = 1.0;
		c1.gridwidth = GridBagConstraints.REMAINDER;
		c1.insets = new Insets(0,10,0,0);
		add(panel, c1);
	
		
		/* Ustalanie wyswietlania wysrodkowanego dla danych symbolicznych */
		((JLabel) table.getDefaultRenderer(String.class)).setHorizontalAlignment(SwingConstants.CENTER);
	}
	
	/**
	 * Ustawia czy prezentowane dane moga byc edytowane.
	 * 
	 * @param edit rowne true wtw gdy ustawiamy mozliwosc edytowania
	 */
	public void setEditable(boolean edit) {
		model.setEditable(edit);
	}
	
	/**
	 * Sprawdza czy zbior danych jest w trakcie edycji.
	 * 
	 * @return true wtw dane sa wlasnie edytowane
	 */
	public boolean isEditing() {
		return table.isEditing();
	}
	
	/**
	 * Powoduje usuniecie zaznaczonych wierszy (elementow) ze zbioru danych.
	 *
	 */
	public void removeSelectedRows() {
		int[] selectedRows = table.getSelectedRows();
		/* Zakladam, ze selectedRows posortowane */
		int tmp = 0;
		for (int i=0; i<selectedRows.length; i++) { 
			model.deleteRow(selectedRows[i] - tmp);
			tmp++;
		}
		
	}
	
	/**
	 * Powoduje dodanie nowego wiersza (elementu) do zbioru danych.
	 *
	 */
	public void addNewRow() {
		model.addNewRow();
	}
	
	/**
	 * Zapisuje aktualnie widoczne w panelu dane w klasie DataManager, 
	 * pytajac przedtem uzytkownika o potwierdzenie.
	 * Gdy sa to dane testowe to pod zmienna DataManager.testData, a
	 * jesli sa to dane treningowe to pod zmienna DataManager.trainData.
	 * 
	 * @return true wtw dane zostaly zapisane.
	 */
	public boolean saveData() {
		String options[] = {"Tak", "Nie"};
		int n = JOptionPane.showOptionDialog(this,
			    "Czy naprawde chcesz zmienic dane "+(testData?"testowe":"treningowe")+"?",
			    "HeaderWriter zmian",
			    JOptionPane.YES_NO_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,     /* nie uzywam niestandardowych ikonek */
			    options,  /* nazwy przyciskow */
			    options[0]);
		
		if (n == 0) { /* Wybrano "Tak" - zapisujemy */
			DoubleDataTable dataTable = model.getData();
			if (testData) {
				DataManager.testData = dataTable;
				DataManager.panel.setTestModified(true);
			}
			else {
				DataManager.trainData = dataTable;
				DataManager.panel.setTrainingModified(true);
			}
			return true;
		}
		else
			return false;

	}
	
	/**
	 * Odpowiednio zmienia wyswietlana tabele z danymi w zaleznosci 
	 * od wybranych przez uzytkownika atrybutow w liscie wyboru.
	 */
	public void valueChanged(ListSelectionEvent e) {
		int[] selected = list.getSelectedIndices();
		model.setChosen(selected);
	}
	
	/**
	 * Model danych dla tabeli wyswietlajacej dane testowe lub treningowe. 
	 * 
	 * @author Michal Manski
	 *
	 */
	class DoubleDataTableModel extends AbstractTableModel {
		/** Zbior danych uzywany przez ten model. */
		protected ArrayList data;
		/** Nazwy kolumn uzywane przez ten model. */
		protected ArrayList columnNames;
		/** Numery kolumn do wyswietlenia (wybrane przez uzytkownika). */ 
		protected int[] chosenColumns;
		/** Okresla czy dane moga byc edytowane */
		protected boolean editable = false;
		
		/**
		 * Konstruktor
		 * 
		 * @param array zbior danych w postaci listy list.
		 * @param columns nazwy kolumn w postaci listy.
		 * @param chosen numery wybranych kolumn.
		 */
		public DoubleDataTableModel(ArrayList array, ArrayList columns, int[] chosen) {
			data = array;
			columnNames = columns;
			chosenColumns = chosen;
		}
		
		/**
		 * Ustawia kolumny, ktore sa wyswietlane na ekranie.
		 * 
		 * @param chosen numery kolumn, ktore maja byc wyswietlane.
		 */
		public void setChosen(int[] chosen) {
			chosenColumns = chosen;
			/* Informuje, ze zmieniono ilosc wyswietlanych kolumn. */
			fireTableStructureChanged();
		}
		
		/** Przeksztalca wybrany przez uzytkownika numer kolumny na rzeczywisty numer tej kolumny. */
		private int column(int col) {
			return chosenColumns[col];	
		}
		
		/** Zwraca liczbe kolumn. */
		public int getColumnCount() {
			return chosenColumns.length;
        }

		/** Zwraca liczbe wierszy (obiektow danych) w modelu. */
        public int getRowCount() {
            return data.size();
        }

        /** Zwraca nazwe odpowiedniej kolumny. */
        public String getColumnName(int col) {
            return (String) columnNames.get(column(col));
        }

        /** Zwraca wartosc odpowiedniej komorki tabeli. */
        public Object getValueAt(int row, int col) {
            return ((ArrayList) data.get(row)).get(column(col));
        }

        /**
         * Zwraca klase reprezentowana przez obiekty w kolumnie col.
         */
        public Class getColumnClass(int col) {
            return getValueAt(0, col).getClass();
        }

        /**
         * Zwraca wartosc zmiennej editable (bo mozna w danej chwili edytowac albo wszystko albo nic).
         */
        public boolean isCellEditable(int row, int col) {
            return editable;
        }

        /**
         * Ustawia wartosc komorki tabeli o numerze wiersz row i 
         * numerze kolumny col.
         */
        public void setValueAt(Object value, int row, int col) {
       
            ((ArrayList) data.get(row)).set(column(col), value);
            fireTableCellUpdated(row, col);

        }
        
        /** Ustawia zmienna okreslajaca czy dane moga byc edytowane. */
        public void setEditable(boolean edit) {
        	editable = edit;
        }
        
        /** Kasuje wiersz o zadanym indeksie. */
        public void deleteRow(int index) {
        	data.remove(index);
        	fireTableRowsDeleted(index,index);
        }
        
        /** Dodaje nowy wiersz do zbioru danych. */
        public void addNewRow() {
        	ArrayList tmpArray = new ArrayList();
			for (int i=0; i<header.noOfAttr(); i++) {
				if ((header.isNominal(i)) || (header.attribute(i) instanceof NominalAttribute)) {
					tmpArray.add("");
				}
				else {
					tmpArray.add(new Double(0));
				}
			}
			data.add(tmpArray);
			fireTableRowsInserted(data.size()-1,data.size()-1);
        }
        
        /** 
         * Zwraca zbior danych reprezentowany przez ten model przekonwertowany
         * do postaci DoubleDataTable.
         */
        public DoubleDataTable getData() {
        	/* Gdy zbior pusty to zwracam null. */
        	if (data.size() == 0)
        		return null;
        	
        	DoubleDataTable dataTable = new ArrayListDoubleDataTable(header);
        	Iterator iter = data.iterator();
        	while (iter.hasNext()) {
        		ArrayList textObject = (ArrayList) iter.next();
        		DoubleData object = new DoubleDataObject(header);
        		for (int i=0; i<header.noOfAttr(); i++) {
        			if (header.isNominal(i) || (header.attribute(i) instanceof NominalAttribute))
        				/* Ten drugi warunek potrzebny, gdy atrybut nominalny tekstowy */
                    {
                        String value = (String) textObject.get(i);
                        double doubleValue = ((NominalAttribute)header.attribute(i)).globalValueCode(value);
                        object.set(i, doubleValue);
                    }
                    else {
                    	Double value = (Double) textObject.get(i);
                        object.set(i, value.doubleValue());
                    }
                }
        		dataTable.add(object);
        	}
        	
        	return dataTable;
        }
	}
}
