/*
 * Created on 2005-04-06
 *
 */
package vr.visual.data;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;

/**
 * Abstrakcyjna klasa reprezentujaca panel pozwalajacy edytowac 
 * dane. Edytowane dane sa wyswietlane za pomoca panelu dataPanel.
 * 
 * @author Michal Manski
 *
 */
public abstract class EditorPanel extends JPanel implements ActionListener {
	/** Panel wyswietlajacy dane */
	PresentationPanel dataPanel;
	/** Przycisk usuwajacy zaznaczone wiersze */
	JButton removeButton;
	/** Przycick dodajacy nowy wiersz */
	JButton addButton;
	/** Przycisk pozwalajacy zaczac edycje danych */
	JButton startButton;
	/** Przycisk do zatwierdzania zmian */
	JButton saveButton;
	/** Panel zawierajacy przyciski */
	JPanel panel;
	
	/**
	 * Konstruktor
	 * 
	 * @param DEL_BUTTON nazwa przycisku, ktory odpowiada za 
	 * kasowanie elementow ze zbioru danych
	 * @param NEW_BUTTON nazwa przycisku, ktory odpowiada za 
	 * dodanie nowego elementu do zbioru danych
	 * @param START_BUTTON nazwa przycisku, ktory odpowiada za 
	 * zapisanie edytowanych danych w "globalnej" pamieci
	 */
	public EditorPanel(String DEL_BUTTON, String NEW_BUTTON, String START_BUTTON) {
		super(new BorderLayout());
		removeButton = new JButton(DEL_BUTTON);
		removeButton.addActionListener(this);
		addButton = new JButton(NEW_BUTTON);
		addButton.addActionListener(this);
		startButton = new JButton(START_BUTTON);
		startButton.addActionListener(this);
		saveButton = new JButton("Zatwierdz zmiany");
		saveButton.addActionListener(this);
		panel = new JPanel();
		panel.add(startButton);
		
	}
	
	/**
	 * 
	 * Odpowiada za wykonanie odpowiednich czynnosci po wcisnieciu
	 * przez uzytkownika przycisku kasowania lub dodawania elementow do 
	 * zbioru danych oraz zapisu danych.
	 * 
	 */
	public void actionPerformed(ActionEvent actEvt) {
		JButton evt = (JButton) actEvt.getSource();
		if (evt == startButton) {
			/* Zaczecie edycji */
			remove(panel);
			panel = new JPanel();
			panel.add(removeButton);
			panel.add(addButton);
			panel.add(saveButton);
			add(panel, BorderLayout.SOUTH);
			dataPanel.setEditable(true);
			updateUI();
		}
		else
			if (evt == addButton) {
				/* Dodawanie nowego wiersza */
				dataPanel.addNewRow();
			}
			else
				if ((evt == removeButton) && (!dataPanel.isEditing())) {
					/* Kasowanie zaznaczonych wierszy */
					dataPanel.removeSelectedRows();
					
				}
				else
					if ((evt == saveButton) && (!dataPanel.isEditing())) {
						/* Zapisujemy zmiany */
						boolean saved = dataPanel.saveData();
						
						/* Jesli rzeczywiscie dane zostaly zapisane, to powrot */ 
						/* do poczatkowego ekranu wyswietlania.                */
						if (saved) {
							remove(panel);
							panel = new JPanel();
							panel.add(startButton);
							add(panel, BorderLayout.SOUTH);
							dataPanel.setEditable(false);
							updateUI();
						}
						
					}
	}
}

