/*
 * Created on 2005-04-05
 *
 */
package vr.visual.data;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import rseslib.structure.attribute.ArrayHeader;
import rseslib.structure.attribute.Attribute;
import rseslib.structure.attribute.Header;
import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.attribute.NumericAttribute;
import vr.system.DataManager;


/**
 * Panel wyswietlajacy naglowek danych.
 * 
 * @author Michal Manski
 *
 */
public class HeaderPresentationPanel extends PresentationPanel {
	/** Tabela wyswietlajaca dane z naglowka. */
	protected JTable table;
	/** Model danych dla tabeli. */
	protected HeaderModel model;
	/** Nazwa atrybutu symbolicznego. */
	final static protected String NOMINAL = "symboliczny";
	/** Nazwa atrybutu numerycznego. */
	final static protected String NUMERIC = "numeryczny";
	/** Nazwa atrybutu tekstowego. */
	final static protected String TEXT = "tekstowy";
	/** Nazwa atrybutu decyzyjnego. */
	final static protected String DECISION = "decyzyjny";
	/** Nazwa atrybutu warunkowego. */
	final static protected String CONDITIONAL = "warunkowy";
	
	/**
	 * Konstruktor
	 * 
	 * @param header naglowek danych.
	 */
	public HeaderPresentationPanel(Header header) {
		if (header == null)
			header = new ArrayHeader(new Attribute[0], null);
		
		ArrayList array = new ArrayList();
		ArrayList tmpArray;
		
		/* Tworze liste list zawierajaca dane z naglowka. */
		for (int i=0; i<header.noOfAttr(); i++) {
			Attribute attr = header.attribute(i);
			tmpArray = new ArrayList();
			tmpArray.add(0, attr.name());
			if (header.isDecision(i)) {
				tmpArray.add(1, DECISION);
			}
			else
				if (header.isConditional(i))
					tmpArray.add(1, CONDITIONAL);
				else
					tmpArray.add(1, TEXT);
			if (header.isNominal(i)) {
				tmpArray.add(2, NOMINAL);
			}
			else
				if (header.isNumeric(i))
					tmpArray.add(2, NUMERIC);
				else  /* Atrybut tekstowy */
					if (header.attribute(i) instanceof NominalAttribute)
						tmpArray.add(2, NOMINAL);
					else
						tmpArray.add(2, NUMERIC);
			array.add(tmpArray);
		}
		
		model = new HeaderModel(array);
		table = new JTable(model);
		table.setPreferredScrollableViewportSize(new Dimension(400, 300));
		
		TableColumn column = null;
		int x;
		for (int i = 0; i < 3; i++) {
		    column = table.getColumnModel().getColumn(i);
		    if (i == 0) x = 200;
			else x = 100;
		    column.setPreferredWidth(x);
		    column.setMinWidth(x);
		}
		setLayout(new BorderLayout());
		add(new JScrollPane(table), BorderLayout.CENTER);
		createComboBoxes();
	}
	
	/**
	 * Tworzy komponenty typu JComboBox sluzace do wyboru typu 
	 * atrybutu w komorkach tabeli.
	 *
	 */
	protected void createComboBoxes() {
		JComboBox attrTypeCombo = new JComboBox();
		attrTypeCombo.addItem(CONDITIONAL);
		attrTypeCombo.addItem(DECISION);
		attrTypeCombo.addItem(TEXT);
		JComboBox valueTypeCombo = new JComboBox();
		valueTypeCombo.addItem(NOMINAL);
		valueTypeCombo.addItem(NUMERIC);
		TableColumn attrTypeColumn = table.getColumnModel().getColumn(1);
		TableColumn valueTypeColumn = table.getColumnModel().getColumn(2);
		attrTypeColumn.setCellEditor(new DefaultCellEditor(attrTypeCombo));
		valueTypeColumn.setCellEditor(new DefaultCellEditor(valueTypeCombo));
		
	}
	
	/**
	 * Ustawia czy prezentowane dane moga byc edytowane.
	 * 
	 * @param edit rowne true wtw gdy ustawiamy mozliwosc edytowania
	 */
	public void setEditable(boolean edit) {
		model.setEditable(edit);
	}
    
	/**
	 * Sprawdza czy zbior danych jest w trakcie edycji.
	 * 
	 * @return true wtw dane sa wlasnie edytowane
	 */
	public boolean isEditing() {
		return table.isEditing();
	}
	
	/**
	 * Powoduje usuniecie zaznaczonych wierszy (atrybutow z naglowka danych).
	 *
	 */
	public void removeSelectedRows() {
		int[] selectedRows = table.getSelectedRows();
		int tmp = 0;
		for (int i=0; i<selectedRows.length; i++) { 
			/* Zakladam ze posortowane */
			model.deleteRow(selectedRows[i] - tmp);
			tmp++;
		}
		
	}
	
	/**
	 * Powoduje dodanie nowego wiersza (atrybutu do naglowka danych).
	 *
	 */
	public void addNewRow() {
		model.addNewRow();
	}
	
	/**
	 * Zapisuje aktualnie widoczne w panelu dane w klasie DataManager,
	 * pytajac przedtem uzytkownika o potwierdzenie.
	 * Zapisywane sa one pod zmienna DataManager.header. Ponadto sa
	 * resetowane dane treningowe i dane testowe zapisane w DataManager.
	 * 
	 * @return true wtw naglowek zostal zapisany.
	 */
	public boolean saveData() {
		String options[] = {"Tak", "Nie"};
		int n = JOptionPane.showOptionDialog(this,
			    "Czy naprawde chcesz zmienic naglowek  danych?\n"+
				"Dane treningowe i testowe zostana wowczas zresetowane.",
			    "HeaderWriter zmian",
			    JOptionPane.YES_NO_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,     /* nie uzywam niestandardowych ikonek */
			    options,  /* nazwy przyciskow */
			    options[0]);
		
		if (n == 0) {  /* Wybrano "Tak" - zapisujemy */
			Header header = model.getData();
			DataManager.dataHeader = header;
			DataManager.trainData = null;
			DataManager.testData = null;
			DataManager.panel.setHeaderModified(true);
			DataManager.panel.setTrainingFile(null);
			DataManager.panel.setTrainingModified(false);
			DataManager.panel.setTestFile(null);
			DataManager.panel.setTestModified(false);
			DataManager.chosenClassifiers = new ArrayList();
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Model danych dla tabeli wyswietlajacej naglowek danych. 
	 * 
	 * @author Michal Manski
	 *
	 */
	public class HeaderModel extends AbstractTableModel {
		/** Zbior danych uzywany przez ten model. */
		protected ArrayList data;
		/** Nazwy kolumn uzywane przez ten model. */
		protected String[] columnNames = {"Nazwa atrybutu",
										  "Rodzaj atrybutu",
                					      "Typ atrybutu"};
		/** Okresla czy dane moga byc edytowane */
		protected boolean editable = false;
		
		/**
		 * Konstruktor
		 * 
		 * @param array naglowek danych w postaci listy list.
		 */
		public HeaderModel(ArrayList array) {
			data = array;
		}
		
		/** Zwraca liczbe kolumn. */
		public int getColumnCount() {
            return columnNames.length;
        }

		/** Zwraca liczbe wierszy w modelu (atrybutow w naglowku). */
        public int getRowCount() {
            return data.size();
        }
        
        /** Zwraca nazwe odpowiedniej kolumny. */
        public String getColumnName(int col) {
            return columnNames[col];
        }

        /** Zwraca wartosc odpowiedniej komorki tabeli. */
        public Object getValueAt(int row, int col) {
            return ((ArrayList) data.get(row)).get(col);
        }

        /**
         * Zwraca klase reprezentowana przez obiekty w kolumnie col.
         */
        public Class getColumnClass(int col) {
            return getValueAt(0, col).getClass();
        }

        /**
         * Zwraca wartosc zmiennej editable (bo mozna w danej chwili edytowac albo wszystko albo nic).
         */
        public boolean isCellEditable(int row, int col) {
            return editable;
        }

        /**
         * Ustawia wartosc komorki tabeli o numerze wiersz row i 
         * numerze kolumny col.
         */
        public void setValueAt(Object value, int row, int col) {
       
            ((ArrayList) data.get(row)).set(col, value);
            fireTableCellUpdated(row, col);

        }
        
        /** Ustawia zmienna okreslajaca czy dane moga byc edytowane. */
        public void setEditable(boolean edit) {
        	editable = edit;
        }
        
        /** Kasuje wiersz o zadanym indeksie. */
        public void deleteRow(int index) {
        	data.remove(index);
        	fireTableRowsDeleted(index,index);
        }
        
        /** Dodaje nowy atrybut do naglowka danych. */
        public void addNewRow() {
        	ArrayList tmpArray = new ArrayList();
			tmpArray.add("");
			tmpArray.add(CONDITIONAL);
			tmpArray.add(NOMINAL);
			
			data.add(tmpArray);
			fireTableRowsInserted(data.size()-1,data.size()-1);
        }
        
        /** 
         * Zwraca naglowek danych reprezentowany przez ten model przekonwertowany
         * do postaci Header.
         */
        public Header getData() {
        	/* Gdy naglowek pusty to zwracam null. */
        	if (data.size() == 0)
        		return null;
        	
        	int i = 0;
        	Attribute attr;
        	byte attrType;
        	Attribute[] attributes = new Attribute[data.size()];
        	Iterator iter = data.iterator();
        	while (iter.hasNext()) {
        		ArrayList textObject = (ArrayList) iter.next();
        		if (textObject.get(1) == CONDITIONAL)
        			attrType = Attribute.ATTR_TYPE_CONDITIONAL;
        		else
        			if (textObject.get(1) == DECISION)
        				attrType = Attribute.ATTR_TYPE_DECISION;
        			else
        				attrType = Attribute.ATTR_TYPE_TEXT;
        		if (textObject.get(2) == NOMINAL)
        			attr = new NominalAttribute(attrType, (String) textObject.get(0));
        		else
        			attr = new NumericAttribute(attrType, (String) textObject.get(0));
        		attributes[i] = attr;
        		i++;
        	}
        	Header newHeader = new ArrayHeader(attributes, null);
        	return newHeader;
        }
        
	}
}
