/*
 * Created on 2005-04-24
 *
 */
package vr.visual.dataIO;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;

import rseslib.processing.classification.TestResult;
import rseslib.structure.attribute.Header;
import rseslib.structure.data.DoubleData;
import rseslib.structure.data.DoubleDataTable;
import vr.visual.VRWindow;


/**
 * Klasa odpowiedzialna za zapisywanie i drukowanie (OBECNIE PORZUCONE) wynikow ostatniej klasyfikacji
 *
 * @author Marcin Staszczyk
 */
public class ClassificationResultsOutput extends JPanel implements ActionListener{
    public ClassificationResultsOutput()
    {
        try
        {
            jbInit();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    /**
	 * parametry podawane kostruktorowi klasy
	 */
	public static final int PRINTING = 1;
	public static final int FILE = 0;

	/** gdzie maja byc wyprowadzone wyniki klasyfikacji */
	int otype;

	/** naglowek z ostatniej klasyfikacji */
	Header hdr;
	/** dane treningowe z ostatniej klasyfikacji */
	DoubleDataTable trn;
	/** dane testowe z ostatniej klasyfikacji */
	DoubleDataTable tst;
	/** mapa z wynikami ostatniej klasyfikacji */
	Map res;

	/** pole wyboru, czy wypisywac naglowek czy nie */
	JCheckBox header = new JCheckBox("Naglowek danych", false);
	/** pole wyboru, czy wypisywac dane treningowe czy nie */
	JCheckBox training = new JCheckBox("Dane treningowe", false);
	/** pole wyboru, czy wypisywac dane testowe czy nie */
	JCheckBox test = new JCheckBox("Dane testowe", false);
	/** pole wyboru, czy wypisywac wyniki czy nie */
	JCheckBox results = new JCheckBox("Wyniki klasyfikacji", false);

	/** przycisk sterujacy */
	JButton aButton = new JButton("Zapisz");
	/** przycisk sterujacy */
	JButton cButton = new JButton("Anuluj");

	/** okno w ktorym wszystko wyswietlamu */
	JFrame frame = new JFrame("Wybor elementow do zapisu");

    /** selektor pliku do zapisu */
    static public JFileChooser fileChooser = new JFileChooser(".");

	/**
	 * Konstruktor klasy pobierajacy dane i wykonujacy cale zapisywanie.
	 *
	 * @param i	Okresla do kad maja zostac zapisane dane
	 * @param a_hdr Naglowek danych
	 * @param a_trn Dane treningowe
	 * @param a_tst Dane testowe
	 * @param a_res Wyniki ostatniej klasyfikacji
	 */
	public ClassificationResultsOutput(int i, Header a_hdr, DoubleDataTable a_trn, DoubleDataTable a_tst, Map a_res)
	{
		super(new GridBagLayout());

		hdr = a_hdr;
		trn = a_trn;
		tst = a_tst;
		res = a_res;

		otype = i;

		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

		GridBagConstraints c1 = new GridBagConstraints();
        c1.fill = GridBagConstraints.HORIZONTAL;

		if (hdr != null)
		{
			//header.setSelected(true);
			add(header, c1);
		}
		if (trn != null)
		{
			//training.setSelected(true);
			add(training, c1);
		}
		if (tst != null)
		{
			//test.setSelected(true);
			add(test, c1);
		}
		if (res != null)
		{
			results.setSelected(true);
			add(results, c1);
		}

		JPanel panel1 = new JPanel();
		panel1.add(aButton);
		aButton.addActionListener(this);
		panel1.add(cButton);
		cButton.addActionListener(this);
		add(panel1);

		frame.setContentPane(this);
		frame.pack();
		frame.setLocationRelativeTo(VRWindow.mainWindow);
		frame.setVisible(true);



	}


	public void actionPerformed(ActionEvent e) {

		if ( e.getSource() == cButton )
		{
			frame.setVisible(false);
			frame.dispose();
		}

		if ( e.getSource() == aButton )
		{
			frame.setVisible(false);
			frame.dispose();

			switch (otype)
			{
				case PRINTING :
				{
					File file = new File("." + File.pathSeparator + "vr" + File.pathSeparator + "visual" +
							File.pathSeparator + "dataIO" + File.pathSeparator + "printingTEMP.tmp");
					printToFile(file);

					try {
						BufferedReader br = new BufferedReader(new FileReader(file));

						String text = "";
						String line = br.readLine();
						while (line != null)
						{
							text += line;
							text += "\n";
							line = br.readLine();
						}

						//new StringPrinter(text); Projekt porzucony, klasa StringPrinter mialaby umozliwic drukowanie podanego tekstu

					} catch (IOException e1) {
						throw new RuntimeException(e1 + e1.toString());
					}
				}
				default : /* wydruk do pliku */
				{
					fileChooser.setMultiSelectionEnabled(false);
					fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
					fileChooser.rescanCurrentDirectory();

				    int returnVal = fileChooser.showSaveDialog(null);
			        if (returnVal != JFileChooser.APPROVE_OPTION)
			        {
			        	return;
			        }

			        printToFile(fileChooser.getSelectedFile());
				}
			}

		}

	}

	/**
	 * Metoda zapisuje do podanego pliku wybrane przez uzytkownika elementy ostatniej klasyfikacji.
	 *
	 * @param file Plik do ktorego maja zostac zapisane dane
	 */
	private void printToFile(File file)
	{
		if ( ! file.isFile() )
		{
			try {
				file.createNewFile();
			} catch (IOException e1) {
				throw new RuntimeException(e1 + e1.getMessage());
			}
		}

        try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(file));

			if (header.isSelected())
			{
				bw.write("Naglowek danych:");
				bw.newLine();
				bw.newLine();
				hdr.store(bw);
				bw.newLine();
				bw.newLine();
			}
			if (training.isSelected())
			{
				bw.write("Dane treningowe:");
				bw.newLine();
				bw.newLine();
				Iterator iter = trn.iterator();
		        while (iter.hasNext())
		        {
		            ((DoubleData)iter.next()).store(bw);
		        }
		        bw.newLine();
		        bw.newLine();
			}
			if (test.isSelected())
			{
				bw.write("Dane testowe:");
				bw.newLine();
				bw.newLine();
				Iterator iter = tst.iterator();
		        while (iter.hasNext())
		        {
		            ((DoubleData)iter.next()).store(bw);
		        }
		        bw.newLine();
		        bw.newLine();
			}
			if (results.isSelected())
			{
				bw.write("Wyniki klasyfikacji:");
				bw.newLine();
				bw.newLine();

				printMap("", res, bw);
			}

			bw.close();

		} catch (Exception e1) {
			throw new RuntimeException(e1 + e1.getMessage());
		}
	}

	/**
	 * Zaisuje mape z wynikami do podanego elementu docelowego
	 *
	 * @param prefix Napis od ktorego ma zaczynac sie nazwa elementu w mapie
	 * @param map	 Mapa w wynikami klasyfikacji
	 * @param bw	 Obiekt odpowiedzialny za fizyczny zapis do pliku
	 */
	private void printMap(String prefix, Map map, BufferedWriter bw) throws IOException
	{
		Iterator iter = map.keySet().iterator();
        while (iter.hasNext())
        {
            String attribute = (String)iter.next();
            Object value = map.get(attribute);

            if (value instanceof TestResult)
            {
            	bw.write(prefix + attribute);
            	bw.newLine();
            	bw.newLine();
            	bw.write(((TestResult)value).toString());
            	bw.newLine();
            	bw.newLine();
            }

            if (value instanceof Map)
            {
            	printMap(prefix + attribute + ".", (Map)value, bw);
            }

        }
	}

    private void jbInit() throws Exception
    {
    }

}
