/*
 * Created on 2005-02-28
 */
package vr.visual.graph;


import java.awt.CardLayout;
import java.util.TreeMap;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.TableXYDataset;

import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.attribute.NumericAttribute;

/**
 * DrawChartSymbNum tworzy i rysuje diagram typu
 * symboliczne * numeryczne
 *  
 * @author Damian Manski
 */
public class DrawChartSymbNum extends JPanel {

    /** nazwa wykresu procentowego*/
    private static final String NORMAL_GRAPH_TEXT = "Wykres procentowy";
    /** nazwa wykresu ilosciowego */
    private static final String PERCENT_GRAPH_TEXT = "Wykres ilociowy";
    
    /** ogolne dane niezalene od accuracy */ 
    TreeMap chartData;
    /** atrybut na osi X */
    NumericAttribute atrX;
    /** atrybut na osi Y */
    NominalAttribute atrY;
    /** tytul wykresu */
    String chartTitle;
    /** accuracy, dla ktorego s narysowane aktualne wykresy */
    private double acc;
    
    /** wykres ilociowy */
    private JFreeChart chart;
    /** Wykres procentowy */
    private JFreeChart chartPercent;
    
    /** panel opakowujcy wykres ilociowy */
    private ChartPanel panel;
    /** panel opakowujcu wykres procentowy */
    private ChartPanel panelPercent;
    /** panel aktualnie wywietlanego wykresu */ 
    private ChartPanel currentChart;
    
    
    /**
     * Konstruktor tworzy diagram num*symb, na podstawie
     * podanych danych w postaci TreeMap. Diagram bdzie tworzony
     * wedlug podanej dokladnosci accuracy i dla zadanych atrybutow 
     * na poszczegolnych osiach. Domylnie zostanie wywietlony wykres 
     * ilociowy 
     * 
     * @param data dane na podstawie ktorych tworzony jest wykres
     * @param aX atrybut numeryczny, ktorego wartoci bd reprezentowane 
     * na osi X
     * @param aY atrybut symboliczny, ktorego wartoci bd reprezentowane 
     * na osi Y
     * @param title tytul wykresu
     * @param accuracy poczatkowa dokladnoc wykresow
     */
    public DrawChartSymbNum(TreeMap data, NumericAttribute aX,
            NominalAttribute aY, String title, double accuracy)
    {
        chartData = data;
        chartTitle = title;
        atrX = aX;
        atrY = aY;
        
        createCharts(accuracy);
        
        /* stworzenie paneli i wywietlenie wykresu ilociowego */
        panel = new ChartPanel(chart);
        panelPercent = new ChartPanel(chartPercent);
        CardLayout crd = new CardLayout();
        this.add(panel);
        this.add(panelPercent);
        this.setLayout(crd);
        crd.addLayoutComponent(panel, NORMAL_GRAPH_TEXT);
        crd.addLayoutComponent(panelPercent, PERCENT_GRAPH_TEXT);
        crd.show(this, NORMAL_GRAPH_TEXT);
        currentChart = panel;
        
    }
    
    /**
     * Najpierw konwertuje wedlug podanej dokladnoci dane do 
     * postaci, ktora jest potrzebna do stworzenia wykresu 
     * typu num*symb czyli TableXYDataset, a nastpnie na ich podstawie
     * tworzy wykresy (ilociowy i procentowy)
     * 
     * @param accuracy
     */
    private void createCharts(double accuracy)
    {
        acc = accuracy;
        
        /* przetworzenie danych */
        TableXYDataset dataset = 
            SymbNumChartDataProvider.generateData(chartData, 
                    atrX, atrY, acc);
        TableXYDataset percentDataset =
            SymbNumChartDataProvider.convertData(dataset);
        
        /* narysowanie wykresow */
        chart = ChartFactory.createStackedXYAreaChart(chartTitle, 
                atrX.name(), atrY.name(), 
            	dataset, PlotOrientation.VERTICAL, true, true, false);
        chartPercent = ChartFactory.createStackedXYAreaChart(chartTitle, 
                atrX.name(), atrY.name(), 
            	percentDataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = (XYPlot) chartPercent.getPlot();
        plot.getRangeAxis().setUpperBound(101);
        
    }
    
    /**
     * Zmiana dokladnoci wykresow, co skutkuje ponownym 
     * przetworzeniem danych i przerysowaniem obu wykresow
     * 
     * @param accuracy
     */
    public void changeAccuracy(double accuracy)
    {
        createCharts(accuracy);
       
        panel.setChart(chart);
        panelPercent.setChart(chartPercent);
        
    }
    
    /**
     * Wyswietla diagram procentowy
     */
    public void drawPercent()
    {
        ((CardLayout) this.getLayout()).show(this, PERCENT_GRAPH_TEXT);
        currentChart = panelPercent;
        
        this.updateUI();
        
    }
    
    /**
     * Wyswietla diagram ilociowy
     */
    public void draw()
    {
        ((CardLayout) this.getLayout()).show(this, NORMAL_GRAPH_TEXT);
        currentChart = panel;
        
        this.updateUI();
        
    }
    
    /**
     * Zwraca panel zawierajcy aktualnie wywietlany wykres
     * 
     * @return aktualnie widoczny panel z wykresem
     */
    public ChartPanel getCurrentChartPanel()
    {
        return currentChart;
    }
}
