/*
 * Created on 2005-02-28
 *
 */
package vr.visual.graph;

import java.util.Iterator;

import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

import rseslib.structure.attribute.Header;
import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.attribute.NumericAttribute;
import rseslib.structure.data.DoubleData;
import rseslib.structure.data.DoubleDataTable;

/**
 * NumNumSymbChartDataProvider zajmuje sie przetwarzaniem danych
 * na potrzeby wykresu typu numeryczne*numeryczne*symboliczne
 *  
 * @author Damian Manski
 */
public class NumNumSymbChartDataProvider {

    /** etykieta wartosci brakujacych na wykresach */
    static private final String MISSING_TEXT = "missing value";
    
    /**
     * Przetwarza dane do formatu z ktorego mozna stworzyc wykres 
     * num*num*symb
     * 
     * @param data zbior obiektow danych do przetworzenia na potrzeby wykresu
     * @param paramX numer atrybutu ze zbioru data, ktory bedzie na osi X
     * @param paramY numer atrybutu ze zbioru data, ktory bedzie na osi Y
     * @param paramV numer atrybutu ze zbioru data, ktorego rozklad badamy
     * @return zbior danych typu XYDataset, potrzebny 
     * do wygenerowania wykresu
     */
    public static XYDataset generateData(DoubleDataTable data, 
            int paramX, int paramY, int paramV)
    {
        XYSeriesCollection dataset = new XYSeriesCollection ();
        Iterator iter = data.iterator();
        Header header = data.attributes();
        NumericAttribute atrX = (NumericAttribute) header.attribute(paramX);
        NumericAttribute atrY = (NumericAttribute) header.attribute(paramY);
        NominalAttribute atrV = (NominalAttribute) header.attribute(paramV);
        
        XYSeries series[] = new XYSeries[atrV.noOfValues() + 1];
        for (int i = 0; i < atrV.noOfValues(); i++)
            series[i] = new XYSeries(atrV.stringValue(atrV.globalValueCode(i)),
                    	false, true);
        series[atrV.noOfValues()] = new XYSeries(MISSING_TEXT, false, true);
        
        boolean missingOnV = false;
        while (iter.hasNext())
        {
            DoubleData row = (DoubleData) iter.next();
            double gV = row.get(paramV);
            double gX = row.get(paramX);
            double gY = row.get(paramY);
            if (!Double.isNaN(gX) && !Double.isNaN(gY)) 
            {
                if (Double.isNaN(gV)) {
                    missingOnV = true;
                    series[atrV.noOfValues()].
                		add(gX, gY);
                }
                else
                    series[atrV.localValueCode(gV)].
            			add(gX, gY);
            }
        }
        
        for (int i = 0; i < atrV.noOfValues(); i++)
            dataset.addSeries(series[i]);
        if (missingOnV)
            dataset.addSeries(series[atrV.noOfValues()]);
        
        return dataset;
        
    }
}
