/*
 * Created on 2005-04-02
 *
 */
package vr.visual.graph;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;

import org.jfree.chart.ChartPanel;
import org.jfree.data.xy.XYDataset;

import rseslib.structure.attribute.Header;
import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.attribute.NumericAttribute;
import rseslib.structure.data.DoubleDataTable;


/**
 * Panel wyswietlajacy wykres typu numeryczne*numeryczne*symboliczne.
 *  
 * @author Damian Manski
 */
public class NumNumSymbChartPanel extends JPanel 
	implements ActionListener {
    
    /** wewnetrzny panel z wyswietlanym wykresem */
    private DrawChartNumNumSymb chartPanel;
    /** przycisk drukowania wykresu */
    private JButton printButton;
    /** nazwa przycisku drukowania */
    static private final String PRINT_TEXT = "Drukuj wykres";
    
    /**
     * Konstruktor tworzy panel zawierajacy wykres num*num*symb 
     * 
     * @param dataTable surowe dane, na podstawie ktorych bedzie stworzony wykres
     * @param iatr1 numer atrybutu w naglowku dataTable, ktory bedzie na osi X
     * @param iatr2 numer atrybutu w naglowku dataTable, ktory bedzie na osi Y
     * @param iatrV numer atrybutu w naglowku dataTable, ktorego rozklad 
     * bedzie badany
     */
    public NumNumSymbChartPanel(DoubleDataTable dataTable, int iatr1, 
            int iatr2, int iatrV) 
    {
        super(new BorderLayout());
        

        Header header = dataTable.attributes();   
        NumericAttribute natr1 = (NumericAttribute) header.attribute(iatr1);
        NumericAttribute natr2 = (NumericAttribute) header.attribute(iatr2);
        NominalAttribute natrV = (NominalAttribute) header.attribute(iatrV);
        
        XYDataset dataset = NumNumSymbChartDataProvider.generateData(dataTable,iatr2,iatr1,iatrV);
        
        chartPanel = new DrawChartNumNumSymb(dataset,"Diagram num*num*symb",
                natr2.name(),natr1.name());
        chartPanel.setBorder(BorderFactory.createEmptyBorder(20, 20, 20, 20));
       
        printButton = new JButton(PRINT_TEXT);
        printButton.setActionCommand("drukuj");
        printButton.addActionListener(this);
        printButton.setMaximumSize(new Dimension(250,20));
        JPanel printPanel = new JPanel();
        printPanel.add(printButton);
        
        this.add(chartPanel, BorderLayout.CENTER);
        this.add(printPanel, BorderLayout.SOUTH);
        
    }
    
    /** Wydrukowanie wykresu */
    private void printChart()
    {
        ChartPanel chartP = chartPanel.getCurrentChartPanel();
        chartP.createChartPrintJob();
        
    }
    
    /**
     * Obsluguje akcje wygenerowane wewnatrz panelu NumNumSymbChartPanel.
     * 
     * @param evt zdarzenie
     */
    public void actionPerformed(ActionEvent evt) {
        
        JButton source = (JButton)(evt.getSource());
		String s = source.getText();
        if (s == PRINT_TEXT) {
            printChart();
        }
         
     }

}
