/*
 * Created on 2005-02-28
 *
 */
package vr.visual.graph;

import java.util.Iterator;

import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;

import rseslib.structure.attribute.Header;
import rseslib.structure.attribute.NominalAttribute;
import rseslib.structure.data.DoubleData;
import rseslib.structure.data.DoubleDataTable;

/**
 * SymbSymbChartDataProvider zajmuje sie przetwarzaniem danych
 * na potrzeby wykresu typu symboliczne*symboliczne
 *  
 * @author Damian Manski
 */
public class SymbSymbChartDataProvider {

    /** etykieta wartosci brakujacych na wykresach */
    static private final String MISSING_TEXT = "missing value";
    
    /**
     * Przetwarza dane do formatu z ktorego mozna stworzyc wykres
     * symb*symb
     * 
     * @param data zbior obiektow danych do przetworzenia na potrzeby wykresu
     * @param paramX numer atrybutu ze zbioru data, ktory bedzie na osi X
     * @param paramY numer atrybutu ze zbioru data, ktory bedzie na osi Y
     * @return zbior danych typu CategoryDataset potrzebnych do 
     * wygenerowania wykresu
     */
    public static CategoryDataset generateData(DoubleDataTable data, 
            int paramX, int paramY)
    {
        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        Iterator iter = data.iterator();
        Header header = data.attributes();
        NominalAttribute atrX = (NominalAttribute) header.attribute(paramX);
        NominalAttribute atrY = (NominalAttribute) header.attribute(paramY);
        
        /* zsumowanie wystapien wszystkich wartosci */
        int sum[][]= new int[atrX.noOfValues() + 1][atrY.noOfValues() + 1];
      
        boolean missingOnX = false;
        boolean missingOnY = false;
        while (iter.hasNext())
        {
            DoubleData row = (DoubleData) iter.next();
            double gX = row.get(paramX);
            double gY = row.get(paramY);
            if (Double.isNaN(gX)) {
                missingOnX = true;
                if (Double.isNaN(gY)) {
                    missingOnY = true;
                    sum[atrX.noOfValues()][atrY.noOfValues()]++;
                }
                else
                    sum[atrX.noOfValues()][atrY.localValueCode(gY)]++;
            }
            else {
                int valX = atrX.localValueCode(gX);
                int valY = atrY.localValueCode(gY);
                if (Double.isNaN(gY)) {
                    missingOnY = true;
                    sum[valX][atrY.noOfValues()]++;
                }
                else
                    sum[valX][valY]++;
            }
           
           
      
        }
        
        /* dodanie par do dataset */
        for (int i = 0; i < atrX.noOfValues(); i++)
            for (int j = 0; j < atrY.noOfValues(); j++)
            {
                dataset.addValue(sum[i][j],atrY.stringValue(atrY.globalValueCode(j)),
                        atrX.stringValue(atrX.globalValueCode(i)));
            }
        /* dodanie par z wartosciami missing_value, (gdy takie wystapily) */
        if (missingOnY) {
            int j = atrY.noOfValues();
            for (int i = 0; i < atrX.noOfValues(); i++)
                dataset.addValue(sum[i][j],MISSING_TEXT,
                        atrX.stringValue(atrX.globalValueCode(i)));
        }
        if (missingOnX) {
            for (int j = 0; j < atrY.noOfValues(); j++)
                dataset.addValue(sum[atrX.noOfValues()][j],atrY.stringValue(atrY.globalValueCode(j)),
                        MISSING_TEXT);
            if (missingOnY)
                dataset.addValue(sum[atrX.noOfValues()][atrY.noOfValues()],MISSING_TEXT,
                        MISSING_TEXT);
                
        }
        return dataset;
    }
    
    /**
     * Na podstawie zbioru danych z udzialem ilosciowym poszczegolnych 
     * wartosci, tworzy nowy zbior danych, pokazujacy procentowy 
     * udzial poszczegolnych wartosci
     * 
     * @param dataset zbior danych do przetworzenia
     * @return zbior danych do wykresu z rozkladem procentowym
     */
    public static CategoryDataset convertData(CategoryDataset dataset)
    {
        DefaultCategoryDataset newDataset = new DefaultCategoryDataset();
        int n_cols = dataset.getColumnCount();
        int n_rows = dataset.getRowCount();
        double all[] = new double[n_cols];
        for (int i = 0; i < n_cols; i++)
            all[i] = 0;
        for (int i = 0; i < n_rows; i++)
            for (int j = 0; j < n_cols; j++)
                all[j] += dataset.getValue(i,j).doubleValue();
        
     
        for (int i = 0; i < n_rows; i++)
            for (int j = 0; j < n_cols; j++)
            {
                newDataset.addValue(((dataset.getValue(i,j).doubleValue())/all[j])*100,
                        dataset.getRowKey(i), dataset.getColumnKey(j));
            }
        return newDataset;
    }
}
