/*
 * Created on 2005-04-08
 *
 */
package vr.visual.results;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;

import rseslib.processing.classification.TestResult;
import rseslib.structure.attribute.NominalAttribute;


/**
 * Panel pozwalajacy wyswietlac wyniki klasyfikacji.
 *
 * @author Michal Manski
 *
 */
public class ClassificationResultsPanel extends JPanel implements ListSelectionListener {
	/** Tabela wyswietlajaca wyniki klasyfikacji. */
	protected JTable table;
	/** Model danych dla tabeli. */
	protected ClassificationResultsModel model;
	/** Lista wyboru pozwalajaca okreslic, dla ktorych klasyfikatorow wyniki maja byc wyswietlane. */
	protected JList list;

	/**
	 * Przeksztalca otrzymane wyniki klassyfikacji do postaci mapy,
	 * w ktorej kluczami sa nazwy klasyfikatorow (moga byc to
	 * nazwy zlozone tzn nazwa klasyfikatora poprzedzona jednym
	 * lub kilkoma nazwami multiklasyfikatorow, w ktoych ten
	 * klasyfikator sie zawieral). Poczatkowa postac powinna byc "rekurencyjna" mapa zawierajaca
	 * jako klucze napisy, a jako wartosci obiekty typu TestResult
	 * albo mape gdzie klucze to napisy, a wartosci to obiekty
	 * TestResult lub mapy itd.
	 *
	 * @param results poczatkowe wyniki klasyfikacji.
	 * @param normResults czesciowe wyniki klasyfikacji po "normalizacji".
	 * @param name prefiks nazwy, ktory bedzie dodany do nazw z results.
	 * @return otrzymane normResults polaczone ze znormalizowanymi wynikami klasyfikacji mapy results.
	 */
	private Map normalizeResults(Map results, Map normResults, String name) {
		Set set = results.keySet();
		Iterator iter = set.iterator();
		while (iter.hasNext()) {
			String key = (String) iter.next();
			Object object = results.get(key);
			if (object.getClass() == TestResult.class) {
				TestResult t = (TestResult) object;
				normResults.put(name+key, t);
			}
			else
				if (object instanceof Map) {
					normResults = normalizeResults((Map) object, normResults, name+key+" - ");
				}
				else
					throw new RuntimeException("Map with results contains something differenf from 'Map' and 'TestResult'");
		}

		return normResults;
	}

	/**
	 * Konstruktor
	 *
	 * @param resultsMap wyniki klasyfikacji
	 * @param decAttr nominalmy atrybut decyzyjny danych, dla ktorych sa te wyniki klasyfikacji.
	 */
	public ClassificationResultsPanel(Map resultsMap, NominalAttribute decAttr) {
		super(new BorderLayout());

		Map results = normalizeResults(resultsMap, new HashMap(), "");


		Set set = results.keySet();
		String[] classifierNames = new String[set.size()];

		Iterator iter = set.iterator();
		int y=0;
		while (iter.hasNext()) {
			classifierNames[y] = (String) iter.next();
			y++;
		}
		ArrayList columns = new ArrayList();
		ArrayList array = new ArrayList();

		/* Dodaje wiersze z wynikami dla kolejnych wartosci atrybutu decyzyjnego */
		for (int dec=0; dec < decAttr.noOfValues(); dec++) {
			ArrayList row = new ArrayList();
			double globalValue = decAttr.globalValueCode(dec);
			row.add(decAttr.stringValue(globalValue));
			for (int i=0; i < classifierNames.length; i++) {
				double acc = ((TestResult) results.get(classifierNames[i])).getDecAccuracy(globalValue);
				acc = ((double) ((int) (acc*10000)))/100;
				row.add(new String((acc)+"%"));
			}
			array.add(row);
		}
		/* Dodaje wiersz z wynikami calosciowymi */
		ArrayList row = new ArrayList();
		row.add("RAZEM:");
		for (int i=0; i < classifierNames.length; i++) {
			double acc = ((TestResult) results.get(classifierNames[i])).getAccuracy();
			acc = ((double) ((int) (acc*10000)))/100;
			row.add(new String((acc)+"%"));
		}
		array.add(row);

		/* Tworze liste dla naglowka tabeli */
		columns.add("Decyzja");
		for (int i=0; i<classifierNames.length; i++) {
			columns.add(classifierNames[i]);
		}

		list = new JList(classifierNames);

		/* Domyslnie ustawiam ze wszystkie kolumny widoczne */
		int[] selected = new int[classifierNames.length];
		for (int i=0; i < classifierNames.length; i++)
			selected[i] = i;

		/* Tworze model i tabele */
		model = new ClassificationResultsModel(array, columns, selected);
		table = new JTable(model) {
			/* Implementowanie "table header tool tips" */
            protected JTableHeader createDefaultTableHeader() {
                return new JTableHeader(columnModel) {
                    public String getToolTipText(MouseEvent e) {
                        String tip = null;
                        java.awt.Point p = e.getPoint();
                        int index = columnModel.getColumnIndexAtX(p.x);
                        int realIndex = columnModel.getColumn(index).getModelIndex();
                        if (realIndex != 0)
                        	return (String) (columnModel.getColumn(index).getHeaderValue());
                        else
                        	return null;
                    }
                };
            }
		};

		table.setPreferredScrollableViewportSize(new Dimension(100+classifierNames.length*150, 300));

		/* Ustalanie preferowanych szerokosci kolumn */
		TableColumn column = null;
		column = table.getColumnModel().getColumn(0);
	    column.setPreferredWidth(100);
	    column.setMinWidth(50);
		for (int i = 0; i < classifierNames.length; i++) {
		    column = table.getColumnModel().getColumn(i+1);
		    column.setPreferredWidth(150);
		    column.setMinWidth(50);
		}

		/* Tworzenie poszczegolnych komponentow */
		JPanel tablePanel = new JPanel();
		tablePanel.setLayout(new BorderLayout());
		tablePanel.add(new JScrollPane(table), BorderLayout.CENTER);
		tablePanel.setMinimumSize(new Dimension(400, 50));

		list.addListSelectionListener(this);
		list.setSelectedIndices(selected);

		JPanel listPanel = new JPanel();
		listPanel.setLayout(new BorderLayout());
		listPanel.setBorder(BorderFactory.createEmptyBorder(20,50,20,50));
		JPanel napisPanel = new JPanel();
		napisPanel.add(new JLabel("Wybierz klasyfikatory do wyswietlenia"));
		listPanel.add(napisPanel, BorderLayout.NORTH);
		listPanel.add(new JScrollPane(list), BorderLayout.CENTER);
		JPanel namePanel = new JPanel();
		namePanel.add(new JLabel("Wyniki ostatniej klasyfikacji"));
		add(namePanel, BorderLayout.NORTH);
		add(tablePanel, BorderLayout.CENTER);
		add(listPanel, BorderLayout.SOUTH);
		setBorder(BorderFactory.createEmptyBorder(20,20,10,20));
	}

	/**
	 * Odpowiednio zmienia wyswietlana tabele z danymi w zaleznosci
	 * od wybranych przez uzytkownika atrybutow w liscie wyboru.
	 */
	public void valueChanged(ListSelectionEvent e) {
		int[] selected = list.getSelectedIndices();
		model.setChosen(selected);
	}

	/**
	 * Model danych dla tabeli wyswietlajacej wyniki klasyfikacji.
	 *
	 * @author Michal Manski
	 *
	 */
	class ClassificationResultsModel extends AbstractTableModel {
		/** Zbior danych uzywany przez ten model. */
		protected ArrayList data;
		/** Nazwy kolumn uzywane przez ten model. */
		protected ArrayList columnNames;
		/** Numery kolumn do wyswietlenia (wybrane przez uzytkownika). */
		protected int[] chosenColumns;

		/**
		 * Konstruktor
		 *
		 * @param array wyniki klasyfikacji w postaci listy list.
		 * @param columns nazwy kolumn w postaci listy.
		 * @param chosen numery wybranych kolumn.
		 */
		public ClassificationResultsModel(ArrayList array, ArrayList columns, int[] chosen) {
			data = array;
			columnNames = columns;
			chosenColumns = chosen;
		}

		/**
		 * Ustawia kolumny, ktore sa wyswietlane na ekranie.
		 *
		 * @param chosen numery kolumn, ktore maja byc wyswietlane.
		 */
		public void setChosen(int[] chosen) {
			chosenColumns = chosen;
			/* Informuje, ze zmieniono ilosc wyswietlanych kolumn. */
			fireTableStructureChanged();
		}

		/** Przeksztalca wybrany przez uzytkownika numer kolumny na rzeczywisty numer tej kolumny. */
		private int column(int col) {
			if (col == 0)
				return 0;
			else {
				return chosenColumns[col-1]+1;
			}
		}

		/** Zwraca liczbe kolumn. */
		public int getColumnCount() {
            return 1+chosenColumns.length;
        }

		/** Zwraca liczbe wierszy w modelu. */
        public int getRowCount() {
            return data.size();
        }

        /** Zwraca nazwe odpowiedniej kolumny. */
        public String getColumnName(int col) {
            return (String) columnNames.get(column(col));
        }

        /** Zwraca wartosc odpowiedniej komorki tabeli. */
        public Object getValueAt(int row, int col) {
            return ((ArrayList) data.get(row)).get(column(col));
        }

        /**
         * Zwraca klase reprezentowana przez obiekty w kolumnie col.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /** Zwraca zawsze false. */
        public boolean isCellEditable(int row, int col) {
            return false;
        }

	}
}
